@@ -315,21 +315,27 @@ set_asbtn_block_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, AS
 /* ASIcon :                                                         */
 /********************************************************************/
 static void
-free_asicon( ASTile* tile )
+free_asimage_tile( ASTile* tile )
 {
-    if (tile->data.icon)
+    if (tile->data.image.im)
     {
-        safe_asimage_destroy (tile->data.icon);
-        tile->data.icon = NULL ;
+        safe_asimage_destroy (tile->data.image.im);
+        tile->data.image.im = NULL ;
     }
     ASSetTileType(tile,AS_TileFreed);
 }
 
 static int
-set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage **scrap_images, int max_width, int max_height )
+set_asimage_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage **scrap_images, int max_width, int max_height )
 {
     int dst_width, dst_height ;
-    ASImage *im = tile->data.icon ;
+    ASImage *im = tile->data.image.im ;
+	unsigned int slice_x_start = tile->data.image.slice_x_start;
+	unsigned int slice_x_end = tile->data.image.slice_x_end;
+	unsigned int slice_y_start = tile->data.image.slice_y_start;
+	unsigned int slice_y_end = tile->data.image.slice_y_end;
+	Bool do_slice_x = ((slice_x_start != 0 || slice_x_end != 0 )&&get_flags( tile->flags, AS_TileHResize ));
+	Bool do_slice_y = ((slice_y_start != 0 || slice_y_end != 0 )&&get_flags( tile->flags, AS_TileVResize ));
 
     if( im == NULL )
         return 0;
@@ -339,7 +345,7 @@ set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage
 
     /* if( ASTileResizeable( *tile ) ) */
     {
-        if( get_flags( tile->flags, AS_TileHScale ) )
+        if( get_flags( tile->flags, AS_TileHScale ) || do_slice_x )
             dst_width = max_width ;
 		else if( get_flags( tile->flags, AS_TileHResize ) && max_width < im->width )
 		{
@@ -351,7 +357,7 @@ set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage
 					layer->clip_x = ((int)im->width - max_width) ;
 			}
 		}
-        if( get_flags( tile->flags, AS_TileVScale ) )
+        if( get_flags( tile->flags, AS_TileVScale ) || do_slice_y )
             dst_height = max_height ;
         else if( get_flags( tile->flags, AS_TileVResize ) && max_height < im->height )
 		{
@@ -367,16 +373,21 @@ set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage
 	LOCAL_DEBUG_OUT( "flags = %lX, dst_size = %dx%d, im_size = %dx%d, max_size = %dx%d, clip = %+d%+d", tile->flags, dst_width, dst_height, im->width, im->height, max_width, max_height, layer->clip_x, layer->clip_y );
     if( im->width != dst_width || im->height != dst_height )
     {
-        im = scale_asimage( ASDefaultVisual, im, dst_width, dst_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+        if( do_slice_x || do_slice_y ) 
+			im = slice_asimage2( ASDefaultVisual, im, slice_x_start, slice_x_end, slice_y_start, slice_y_end, dst_width, dst_height, 
+								 get_flags( tile->flags, AS_TileHScale|AS_TileVScale ), ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+		else
+			im = scale_asimage( ASDefaultVisual, im, dst_width, dst_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+
         if( im == NULL )
-            im = tile->data.icon ;
+            im = tile->data.image.im ;
         else
             *scrap_images = im ;
     }
 
     layer->im = im;
-    layer->dst_x = tile->x;
-    layer->dst_y = tile->y ;
+    layer->dst_x = tile->x - layer->clip_x;
+    layer->dst_y = tile->y - layer->clip_y;
     if( ASTileHResizeable(*tile) )
         layer->clip_width  = max_width ;
     else
@@ -532,7 +543,7 @@ struct
 {
 {"Spacer",  NULL, NULL, NULL},
 {"Buttons", free_asbtn_block, check_btn_point, NULL,                  set_asbtn_block_layer},
-{"Icon",    free_asicon,      NULL,            NULL,                  set_asicon_layer},
+{"Image",    free_asimage_tile, NULL,            NULL,                set_asimage_layer},
 {"Label",   free_aslabel,     NULL,            aslabel_style_changed, set_aslabel_layer},
 {"none",    NULL, NULL, NULL},
 {"none",    NULL, NULL, NULL},
@@ -625,6 +636,9 @@ calculate_astbar_height (ASTBarData * tbar)
     if (tbar)
     {
         register int i = tbar->tiles_num ;
+#if	defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
+         print_astbar_tiles(tbar); 
+#endif
         while( --i >= 0 )
             if( ASTileType(tbar->tiles[i]) != AS_TileFreed &&
                 !ASTileIgnoreHeight(tbar->tiles[i]))
@@ -732,14 +746,14 @@ update_astbar_bevel_size (ASTBarData * tbar)
 		if (tbar->style[i])
 		{
             mystyle_make_bevel (tbar->style[i], &bevel, tbar->hilite[i], False);
-			if (tbar->left_bevel < bevel.left_outline)
-				tbar->left_bevel = bevel.left_outline;
-			if (tbar->top_bevel < bevel.top_outline)
-				tbar->top_bevel = bevel.top_outline;
-			if (tbar->right_bevel < bevel.right_outline)
-				tbar->right_bevel = bevel.right_outline;
-			if (tbar->bottom_bevel < bevel.bottom_outline)
-				tbar->bottom_bevel = bevel.bottom_outline;
+			if (tbar->left_bevel < bevel.left_outline + bevel.left_inline)
+				tbar->left_bevel = bevel.left_outline + bevel.left_inline;
+			if (tbar->top_bevel < bevel.top_outline + bevel.top_inline)
+				tbar->top_bevel = bevel.top_outline + bevel.top_inline;
+			if (tbar->right_bevel < bevel.right_outline + bevel.right_inline)
+				tbar->right_bevel = bevel.right_outline + bevel.right_inline;
+			if (tbar->bottom_bevel < bevel.bottom_outline + bevel.bottom_inline)
+				tbar->bottom_bevel = bevel.bottom_outline + bevel.bottom_inline;
 		}
 }
 
@@ -774,6 +788,20 @@ Bool set_astbar_composition_method( ASTBarData *tbar, unsigned int state, unsign
 	return changed;
 }
 
+Bool set_astbar_huesat( ASTBarData *tbar, unsigned int state, int hue, int sat )
+{
+    Bool          changed = False;
+    if (tbar && state < BAR_STATE_NUM)
+	{
+        changed = (tbar->hue[state] != hue || tbar->sat[state] != sat );
+        tbar->hue[state] = hue ;
+		tbar->sat[state] = sat ;
+        if (changed )
+            set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+    }
+	return changed;
+}
+
 
 static inline void
 set_astile_styles( ASTBarData *tbar, ASTile *tile, int state )
@@ -847,7 +875,36 @@ set_astbar_style (ASTBarData * tbar, unsigned int state, const char *style_name)
     return changed;
 }
 
-static ASTile *
+Bool
+invalidate_astbar_style (ASTBarData * tbar, int state)
+{
+	Bool          changed = False;
+
+	if( tbar && state < BAR_STATE_NUM )
+	{
+		if( state < 0 ) 
+		{
+			int i ;
+			for( i = 0 ; i < BAR_STATE_NUM ; ++i ) 
+				if( tbar->back[i] )
+					changed = invalidate_astbar_style (tbar, i);
+		}else if( tbar->style[state] != NULL )
+		{
+			changed = True;
+			tbar->style[state] = NULL;
+			if( tbar->back[state] )
+			{
+				destroy_asimage (&(tbar->back[state]));
+				tbar->back[state] = NULL;
+			}
+			set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+		}
+	}
+    return changed;
+}
+
+
+static int
 add_astbar_tile( ASTBarData *tbar, int type, unsigned char col, unsigned char row, int flip, int align )
 {
     int new_idx = -1;
@@ -866,33 +923,60 @@ add_astbar_tile( ASTBarData *tbar, int type, unsigned char col, unsigned char ro
             tbar->tiles = realloc( tbar->tiles, (((tbar->tiles_num>>2)+1)<<2)*sizeof(ASTile));
         ++(tbar->tiles_num);
     }
-
-    if( get_flags( flip, FLIP_VERTICAL ) )
-        align_flags = (((align&PAD_H_MASK)>>PAD_H_OFFSET)<<PAD_V_OFFSET)|
-                      (((align&PAD_V_MASK)>>PAD_V_OFFSET)<<PAD_H_OFFSET)|
-                      (((align&RESIZE_H_MASK)>>RESIZE_H_OFFSET)<<RESIZE_V_OFFSET)|
-                      (((align&RESIZE_V_MASK)>>RESIZE_V_OFFSET)<<RESIZE_H_OFFSET) ;
-
-    if( get_flags( flip, FLIP_UPSIDEDOWN ) )
-        align_flags = ((align_flags&0x0005)<<1)|((align_flags&(0x0005<<1))>>1)|(align_flags&RESIZE_MASK);
-
 	LOCAL_DEBUG_CALLER_OUT( "type = %d, col = %d, row = %d, flip = %d, align_flags = 0x%lX", type, col, row, flip, align_flags );
 
-	clear_flags( align_flags, FIT_LABEL_SIZE);
+    if( get_flags( flip, FLIP_VERTICAL ) && get_flags( flip, FLIP_UPSIDEDOWN ) )
+	{
+		clear_flags( align_flags, PAD_MASK );
+		if( get_flags( align, PAD_LEFT ) )
+			set_flags( align_flags, PAD_TOP );
+		if( get_flags( align, PAD_RIGHT ) )
+			set_flags( align_flags, PAD_BOTTOM );
+		if( get_flags( align, PAD_TOP ) )
+			set_flags( align_flags, PAD_RIGHT );
+		if( get_flags( align, PAD_BOTTOM ) )
+			set_flags( align_flags, PAD_LEFT );
+	}else if( get_flags( flip, FLIP_VERTICAL ) )
+	{
+		clear_flags( align_flags, PAD_MASK );
+		if( get_flags( align, PAD_LEFT ) )
+			set_flags( align_flags, PAD_BOTTOM );
+		if( get_flags( align, PAD_RIGHT ) )
+			set_flags( align_flags, PAD_TOP );
+		if( get_flags( align, PAD_TOP ) )
+			set_flags( align_flags, PAD_LEFT );
+		if( get_flags( align, PAD_BOTTOM ) )
+			set_flags( align_flags, PAD_RIGHT );
+	}else if( get_flags( flip, FLIP_UPSIDEDOWN) )
+	{
+		clear_flags( align_flags, PAD_MASK );
+		if( get_flags( align, PAD_LEFT ) )
+			set_flags( align_flags, PAD_RIGHT );
+		if( get_flags( align, PAD_RIGHT ) )
+			set_flags( align_flags, PAD_LEFT );
+		if( get_flags( align, PAD_TOP ) )
+			set_flags( align_flags, PAD_BOTTOM );
+		if( get_flags( align, PAD_BOTTOM ) )
+			set_flags( align_flags, PAD_TOP );
+	}
+
 	if( get_flags( flip, FLIP_VERTICAL ) )
 	{
+		clear_flags( align_flags, RESIZE_MASK|FIT_LABEL_SIZE );
+		if( get_flags( align, RESIZE_H ) )
+			set_flags( align_flags, RESIZE_V );
+		if( get_flags( align, RESIZE_V ) )
+			set_flags( align_flags, RESIZE_H );
+		if( get_flags( align, RESIZE_H_SCALE ) )
+			set_flags( align_flags, RESIZE_V_SCALE );
+		if( get_flags( align, RESIZE_V_SCALE ) )
+			set_flags( align_flags, RESIZE_H_SCALE );
 		if( get_flags( align, FIT_LABEL_WIDTH ) )
-        	set_flags( align_flags, FIT_LABEL_HEIGHT );
-    	if( get_flags( align, FIT_LABEL_HEIGHT ) )
-        	set_flags( align_flags, FIT_LABEL_WIDTH );
-	}else
-	{
+			set_flags( align_flags, FIT_LABEL_HEIGHT );
+		if( get_flags( align, FIT_LABEL_HEIGHT ) )
+			set_flags( align_flags, FIT_LABEL_WIDTH );
+	}	 
 
-		if( get_flags( align, FIT_LABEL_WIDTH ) )
-        	set_flags( align_flags, FIT_LABEL_WIDTH );
-    	if( get_flags( align, FIT_LABEL_HEIGHT ) )
-        	set_flags( align_flags, FIT_LABEL_HEIGHT );
-	}
 	LOCAL_DEBUG_CALLER_OUT( "type = %d, flip = %d, align = 0x%X, align_flags = 0x%lX", type, flip, align, align_flags );
     align_flags &= (PAD_MASK|RESIZE_MASK|FIT_LABEL_SIZE);
 
@@ -903,7 +987,7 @@ add_astbar_tile( ASTBarData *tbar, int type, unsigned char col, unsigned char ro
                                  ((flip<<AS_TileFlipOffset)&AS_TileFlipMask)|
                                  ((align_flags<<AS_TileFloatingOffset));
     set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
-    return &(tbar->tiles[new_idx]);
+    return new_idx;
 }
 
 Bool
@@ -937,10 +1021,11 @@ add_astbar_spacer( ASTBarData *tbar, unsigned char col, unsigned char row, int f
 {
     if( tbar )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileSpacer, col, row, flip, align );
+		int idx = add_astbar_tile( tbar, AS_TileSpacer, col, row, flip, align );
+        ASTile *tile = &(tbar->tiles[idx]) ;
         tile->width = width ;
         tile->height = height ;
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
@@ -952,7 +1037,9 @@ add_astbar_btnblock( ASTBarData * tbar, unsigned char col, unsigned char row, in
 {
     if( tbar )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileBtnBlock, col, row, flip, align );
+        int idx = add_astbar_tile( tbar, AS_TileBtnBlock, col, row, flip, align );
+        ASTile *tile = &(tbar->tiles[idx]) ;
+
 /*      if( get_flags( flip, FLIP_VERTICAL ) )
         {
             int tmp = top_margin ; 
@@ -962,7 +1049,7 @@ add_astbar_btnblock( ASTBarData * tbar, unsigned char col, unsigned char row, in
  */ 
         build_btn_block( tile, from_list, context_mask, count, left_margin, top_margin,
                          spacing, order );
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
@@ -972,11 +1059,13 @@ add_astbar_icon( ASTBarData * tbar, unsigned char col, unsigned char row, int fl
 {
     if( tbar && icon )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileIcon, col, row, flip, align );
+        int idx = add_astbar_tile( tbar, AS_TileIcon, col, row, flip, align );
+		ASTile *tile = &(tbar->tiles[idx]) ;
+
         if( flip == 0 )
         {
-            if( (tile->data.icon = dup_asimage( icon )) == NULL )
-                    tile->data.icon = clone_asimage( icon, 0xFFFFFFFF );
+            if( (tile->data.image.im = dup_asimage( icon )) == NULL )
+                    tile->data.image.im = clone_asimage( icon, 0xFFFFFFFF );
         }else
 		{
 			int dst_width = icon->width, dst_height = icon->height ;
@@ -985,23 +1074,46 @@ add_astbar_icon( ASTBarData * tbar, unsigned char col, unsigned char row, int fl
 				dst_width = icon->height ;
 				dst_height = icon->width ;
 			}
-            tile->data.icon = flip_asimage( ASDefaultVisual, icon, 0, 0, dst_width, dst_height, flip, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
+            tile->data.image.im = flip_asimage( ASDefaultVisual, icon, 0, 0, dst_width, dst_height, flip, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
 		}
-        tile->width = tile->data.icon->width;
-        tile->height = tile->data.icon->height;
+        tile->width = tile->data.image.im->width;
+        tile->height = tile->data.image.im->height;
         ASSetTileSublayers(*tile,1);
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
 
+int
+add_astbar_image( ASTBarData * tbar, unsigned char col, unsigned char row, int flip, int align, ASImage *im, 
+				  unsigned short slice_x_start, unsigned short slice_x_end,
+				  unsigned short slice_y_start, unsigned short slice_y_end)
+{
+    int idx = add_astbar_icon( tbar, col, row, flip, align, im);
+	if( idx >= 0 ) 
+	{	
+		ASImageTile *imtile = &(tbar->tiles[idx].data.image) ;
+		imtile->slice_x_start = slice_x_start;
+		imtile->slice_x_end = slice_x_end;
+		imtile->slice_y_start = slice_y_start;
+		imtile->slice_y_end = slice_y_end;
+        
+		return idx;
+	}
+    return -1;
+}
+
+
 int
 add_astbar_label( ASTBarData * tbar, unsigned char col, unsigned char row, int flip, int align, short h_padding, short v_padding, const char *text, unsigned long encoding)
 {
-LOCAL_DEBUG_CALLER_OUT( "label \"%s\"", text?text:"null" );
+LOCAL_DEBUG_CALLER_OUT( "encoding = %ld, label \"%s\"", encoding, text?text:"null" );
     if( tbar )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileLabel, col, row, flip, align );
+		
+        int idx = add_astbar_tile( tbar, AS_TileLabel, col, row, flip, align );
+		ASTile *tile = &(tbar->tiles[idx]) ;
+
         ASLabel *lbl = &(tile->data.label);
         lbl->text = mystrdup(text);
 		lbl->encoding = encoding ;
@@ -1010,7 +1122,7 @@ LOCAL_DEBUG_CALLER_OUT( "label \"%s\"", text?text:"null" );
         set_astile_styles( tbar, tile, -1 );
 
         ASSetTileSublayers(*tile,1);
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
@@ -1020,12 +1132,14 @@ change_astbar_label (ASTBarData * tbar, int index, const char *label, unsigned l
 {
 	Bool          changed = False;
 
-LOCAL_DEBUG_CALLER_OUT( "tbar(%p)->index(%d)->label(\"%s\")", tbar, index, label?label:"null" );
+LOCAL_DEBUG_CALLER_OUT( "tbar(%p)->index(%d)->label(ENC = %ld, TXT=\"%s\")", tbar, index, encoding, label?label:"null" );
     if (tbar && tbar->tiles)
 	{
         ASLabel *lbl = &(tbar->tiles[index].data.label);
-        if( ASTileType(tbar->tiles[index]) != AS_TileLabel )
+		if( ASTileType(tbar->tiles[index]) != AS_TileLabel )
             return False;
+		LOCAL_DEBUG_OUT( "old_label(ENC = %ld, TXT=\"%s\" )", lbl->encoding, lbl->text?lbl->text:"null" );
+        
 
         if (label == NULL)
 		{
@@ -1055,7 +1169,6 @@ LOCAL_DEBUG_CALLER_OUT( "tbar(%p)->index(%d)->label(\"%s\")", tbar, index, label
 	return changed;
 }
 
-
 Bool
 change_astbar_first_label (ASTBarData * tbar, const char *label, unsigned long encoding)
 {
@@ -1272,7 +1385,7 @@ render_astbar (ASTBarData * tbar, ASCanvas * pc)
 	ASImage      *merged_im;
 	int           state;
 	ASAltImFormats fmt = ASA_ScratchXImageAndAlpha;
-    int l ;
+    int l, changed ;
     short col_width[AS_TileColumns] = {0};
     short row_height[AS_TileRows] = {0};
     short col_x[AS_TileColumns] = {0};
@@ -1350,8 +1463,11 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
             good_layers += ASTileSublayers(tbar->tiles[l]);
 			if( !ASTileIgnoreWidth(tbar->tiles[l]) )
 			{
-            	if( col_width[pos] < tbar->tiles[l].width )
-                	col_width[pos] = tbar->tiles[l].width;
+				int w = tbar->tiles[l].width  ;
+				if( w > tbar->width ) 
+					w = tbar->width ;
+            	if( col_width[pos] < w )
+                	col_width[pos] = w;
                 /* floating column has at least one element padded/resizable in horizontal dir,
                  * and no fixed elements */
                 if( !ASTileHFloating(tbar->tiles[l]) )
@@ -1363,8 +1479,11 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
             pos = ASTileRow(tbar->tiles[l]);
 			if( !ASTileIgnoreHeight(tbar->tiles[l]) )
 			{
-            	if( row_height[pos] < tbar->tiles[l].height )
-                	row_height[pos] = tbar->tiles[l].height;
+				int h = tbar->tiles[l].height  ;
+				if( h > tbar->height ) 
+					h = tbar->height ;
+            	if( row_height[pos] < h )
+                	row_height[pos] = h;
                 /* floating row has at least one element padded/resizable in vertical dir,
                  * and no fixed elements */
                 if( !ASTileVFloating(tbar->tiles[l]) )
@@ -1374,8 +1493,10 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
             }
         }
     /* pass 2: see how much space we have left that needs to be floating to some rows/columns : */
-    space_left_x = tbar->width - (h_bevel_size+tbar->h_border*2+bevel.left_inline+bevel.right_inline);
-    space_left_y = tbar->height- (v_bevel_size+tbar->v_border*2+bevel.top_inline+bevel.bottom_inline);
+    LOCAL_DEBUG_OUT( "bar_size = %dx%d", tbar->width, tbar->height );
+
+    space_left_x = tbar->width -(h_bevel_size+tbar->h_border*2+bevel.left_inline+bevel.right_inline);
+    space_left_y = tbar->height-(v_bevel_size+tbar->v_border*2+bevel.top_inline+bevel.bottom_inline);
     LOCAL_DEBUG_OUT( "from: space_left_x = %d, space_left_y = %d", space_left_x, space_left_y );
     for( l = 0 ; l < AS_TileColumns ; ++l )
     {
@@ -1418,6 +1539,26 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
                     break;
             }
         }
+	l = 0 ; changed = 0 ;
+	while( space_left_x < -1 )     /* ohh, no, we need to trim columns to fit */
+	{
+		if( col_width[l] > 0 ) 		
+		{
+			int to_trim = (-space_left_x*col_width[l])/tbar->width ; 
+			if( to_trim == 0 ) 
+				to_trim = 1 ;
+			col_width[l] -= to_trim;
+			space_left_x  += to_trim;
+			++changed ;
+		}	 
+		if( ++l >= AS_TileColumns )
+		{
+			if( changed == 0 ) 
+				break;	  
+			l = 0 ;
+			changed = 0;
+		}
+	}	 
     /* pass 4: now we determine spread padding among affected rows : */
     if( floating_rows_count > 0 && space_left_y != 0)
         for( l = 0 ; l < AS_TileRows ; ++l )
@@ -1439,10 +1580,30 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
                     break;
             }
         }
+	l = 0 ; changed = 0 ;
+	while( space_left_y < -1 )     /* ohh, no, we need to trim rows to fit */
+	{
+		if( row_height[l] > 0 ) 		
+		{
+			int to_trim = (-space_left_y*row_height[l])/tbar->height ; 
+			if( to_trim == 0 ) 
+				to_trim = 1 ;
+			row_height[l] -= to_trim;
+			space_left_y  += to_trim;
+			++changed ;
+		}	 
+		if( ++l >= AS_TileRows )
+		{	
+			if( changed == 0 ) 
+				break;	  
+			l = 0 ;
+			changed = 0;
+		}
+	}	 
 
     /* pass 5: now we determine offset of each row/column : */
     x = bevel.left_outline+bevel.left_inline+tbar->h_border ;
-    y = bevel.top_outline +bevel.top_inline+tbar->v_border ;
+    y = bevel.top_outline+bevel.top_inline+tbar->v_border ;
     for( l = 0 ; l < AS_TileColumns ; ++l )
     {
         col_x[l] = x ;
@@ -1525,7 +1686,26 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
 	}else if (pc->shape)
  		fill_canvas_mask (pc, tbar->win_x, tbar->win_y, tbar->width, tbar->height );
 #endif
-    merged_im = merge_layers (ASDefaultVisual, &layers[0], good_layers, tbar->width, tbar->height, fmt, 0, ASIMAGE_QUALITY_DEFAULT);
+	if( get_flags( ASDefaultVisual->glx_support, ASGLX_UseForImageTx ) )
+		fmt = ASA_ASImage ;
+	if( tbar->hue[state] > 0 || tbar->sat[state] >= 0 )
+	{
+		ASImage *tmp_im = merge_layers (ASDefaultVisual, &layers[0], good_layers, tbar->width, tbar->height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);		
+		if( tmp_im ) 
+		{	
+			merged_im = adjust_asimage_hsv( ASDefaultVisual, tmp_im,
+				    	0, 0, 
+	  			    	tmp_im->width, tmp_im->height,
+						0, 360,
+						tbar->hue[state] < 0? 0: tbar->hue[state], 
+						tbar->sat[state] < 0? 0: tbar->sat[state], 
+						0,
+						fmt,
+						0, ASIMAGE_QUALITY_DEFAULT );
+			destroy_asimage( &tmp_im );
+		}
+	}else	 
+    	merged_im = merge_layers (ASDefaultVisual, &layers[0], good_layers, tbar->width, tbar->height, fmt, 0, ASIMAGE_QUALITY_DEFAULT);
     for( l = 0 ; l < good_layers ; ++l )
         if( scrap_images[l] )
             destroy_asimage( &(scrap_images[l]) );
@@ -1585,14 +1765,18 @@ LOCAL_DEBUG_OUT( "bar's geometry = %dx%d%+d%+d, pointer posish = %+d%+d", tbar->
 }
 
 void
-on_astbar_pointer_action( ASTBarData *tbar, int context, Bool leave )
+on_astbar_pointer_action( ASTBarData *tbar, int context, Bool leave, Bool pointer_moved )
 {
+	static ASBalloon *last_balloon = NULL ;
     LOCAL_DEBUG_CALLER_OUT( "%p, %s, %d", tbar, context2text(context), leave );
-    if( tbar == NULL )
+    
+	if( tbar == NULL )
     {
         tbar = FocusedBar ;
         leave = True ;
     }
+	if( pointer_moved ) 
+		last_balloon = NULL ; 
     LOCAL_DEBUG_OUT( "%p, %s, %d", tbar, context2text(context), leave );
     if( tbar )
     {
@@ -1625,11 +1809,12 @@ on_astbar_pointer_action( ASTBarData *tbar, int context, Bool leave )
             withdraw_balloon( balloon );
             if( tbar == FocusedBar )
                 FocusedBar = NULL ;
-        }else
+        }else if( balloon != last_balloon ) 
         {
             display_balloon( balloon );
             FocusedBar = tbar ;
         }
+		last_balloon = balloon ;
     }
 }
 
@@ -1655,13 +1840,14 @@ set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned lo
                             if( bb->buttons[k].balloon != NULL )
                             {
                                 balloon_set_text (bb->buttons[k].balloon, text, encoding);
-                                LOCAL_DEBUG_OUT( "changed balloon for tbar(%p)->context(0x%X)->button(%d)->text(%s)->balloon(%p)",
-                                                 tbar, context, k, text, bb->buttons[k].balloon );
+                                LOCAL_DEBUG_OUT( "changed balloon for tbar(%p)->context(0x%X)->button(%d)->encoding(%ld)->text(%s)->balloon(%p)",
+                                                 tbar, context, k, encoding, text, bb->buttons[k].balloon
+												  );
                             }else
                             {
                                 bb->buttons[k].balloon = create_asballoon_with_text( tbar, text, encoding );
-                                LOCAL_DEBUG_OUT( "created balloon for tbar(%p)->context(0x%X)->button(%d)->text(%s)->balloon(%p)",
-                                                 tbar, context, k, text, bb->buttons[k].balloon );
+                                LOCAL_DEBUG_OUT( "created balloon for tbar(%p)->ct(0x%X)->btn(%d)->enc(%ld)->text(%s)->balloon(%p)",
+								               tbar, context, k, encoding, text, bb->buttons[k].balloon );
                             }
                             return ;
                         }
