@@ -74,6 +74,22 @@ Atom          _AS_VIRTUAL_ROOT = None;
 Atom          _AS_DESK_NUMBERS = None;
 Atom          _AS_CURRENT_DESK = None;
 Atom          _AS_CURRENT_VIEWPORT = None;
+Atom  		  _AS_SERVICE_WINDOW = None;
+
+Atom  		_AS_TBAR_PROPS		        = None;
+Atom  		_AS_BUTTON_CLOSE	     	= None;
+Atom  		_AS_BUTTON_CLOSE_PRESSED 	= None;
+Atom  		_AS_BUTTON_MAXIMIZE	     	= None;
+Atom  		_AS_BUTTON_MAXIMIZE_PRESSED = None;
+Atom  		_AS_BUTTON_MINIMIZE	     	= None;
+Atom  		_AS_BUTTON_MINIMIZE_PRESSED = None;
+Atom  		_AS_BUTTON_SHADE 			= None;
+Atom  		_AS_BUTTON_SHADE_PRESSED	= None;
+Atom  		_AS_BUTTON_MENU 			= None;
+Atom  		_AS_BUTTON_MENU_PRESSED 	= None;
+
+Atom  		_GTK_READ_RCFILES = None;
+Atom  		_KIPC_COMM_ATOM = None;
 
 /* Crossreferences of atoms into flag value for
    different atom list type of properties :*/
@@ -122,7 +138,26 @@ AtomXref  _WMPropAtoms[] = {
 	{"_AS_DESK_NUMBERS", &_AS_DESK_NUMBERS},   /* translation of the continuous range of */
     {"_AS_CURRENT_DESK", &_AS_CURRENT_DESK},   /* current afterstep desk */
     {"_AS_CURRENT_VIEWPORT", &_AS_CURRENT_VIEWPORT},   /* current afterstep viewport */
-    {NULL, NULL, 0, None}
+    {"_AS_SERVICE_WINDOW", &_AS_SERVICE_WINDOW},   /* current afterstep Scr.ServiceWin */
+    
+#define WMPROPS_ATOM_DESC(a)   { #a, &a }
+
+    WMPROPS_ATOM_DESC(_AS_TBAR_PROPS),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_CLOSE),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_CLOSE_PRESSED),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MAXIMIZE),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MAXIMIZE_PRESSED),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MINIMIZE),
+	WMPROPS_ATOM_DESC(_AS_BUTTON_MINIMIZE_PRESSED),
+	WMPROPS_ATOM_DESC(_AS_BUTTON_SHADE),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_SHADE_PRESSED),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MENU),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MENU_PRESSED),
+	
+	WMPROPS_ATOM_DESC(_GTK_READ_RCFILES),
+	WMPROPS_ATOM_DESC(_KIPC_COMM_ATOM),
+	
+	{NULL, NULL, 0, None}
 };
 
 AtomXref  *WMPropAtoms = &_WMPropAtoms[0];
@@ -259,13 +294,23 @@ setup_compatibility_props (ScreenInfo * scr)
 {
 	/* Old Gnome Compatibility */
 	set_multi32bit_property (scr->Root, _XA_WIN_PROTOCOLS, XA_ATOM, 7,
-							 _XA_WIN_LAYER, _XA_WIN_STATE, _XA_WIN_WORKSPACE, _XA_WIN_HINTS,
-							 _XA_WIN_WORKSPACE_COUNT, _XA_WIN_CLIENT_LIST,_XA_WIN_DESKTOP_BUTTON_PROXY);
+							 _XA_WIN_LAYER, 				_XA_WIN_STATE, 				_XA_WIN_WORKSPACE, 
+							 _XA_WIN_HINTS, 				_XA_WIN_WORKSPACE_COUNT, 	_XA_WIN_CLIENT_LIST,
+							 _XA_WIN_DESKTOP_BUTTON_PROXY);
 	/* Ext WM compatibility */
-	set_multi32bit_property (scr->Root, _XA_NET_SUPPORTED, XA_ATOM, 28, _XA_NET_SUPPORTED, _XA_NET_CLIENT_LIST, _XA_NET_CLIENT_LIST_STACKING, _XA_NET_NUMBER_OF_DESKTOPS, _XA_NET_DESKTOP_GEOMETRY, _XA_NET_DESKTOP_VIEWPORT, _XA_NET_CURRENT_DESKTOP, _XA_NET_ACTIVE_WINDOW, _XA_NET_SUPPORTING_WM_CHECK, _XA_NET_VIRTUAL_ROOTS,	/* 11 */
+	set_multi32bit_property (scr->Root, _XA_NET_SUPPORTED, XA_ATOM, 29, 
+							 _XA_NET_SUPPORTED, 			_XA_NET_CLIENT_LIST, 		_XA_NET_CLIENT_LIST_STACKING, 
+							 _XA_NET_NUMBER_OF_DESKTOPS, 	_XA_NET_DESKTOP_GEOMETRY, 	_XA_NET_DESKTOP_VIEWPORT, 
+							 _XA_NET_CURRENT_DESKTOP, 		_XA_NET_ACTIVE_WINDOW, 		_XA_NET_SUPPORTING_WM_CHECK, 
+							 _XA_NET_VIRTUAL_ROOTS,	/* 10 */
 							 /* client window properties : */
-							 _XA_NET_WM_NAME, _XA_NET_WM_DESKTOP, _XA_NET_WM_WINDOW_TYPE, _XA_NET_WM_WINDOW_TYPE_DESKTOP, _XA_NET_WM_WINDOW_TYPE_DOCK, _XA_NET_WM_WINDOW_TYPE_TOOLBAR, _XA_NET_WM_WINDOW_TYPE_MENU, _XA_NET_WM_WINDOW_TYPE_DIALOG, _XA_NET_WM_WINDOW_TYPE_NORMAL, _XA_NET_WM_STATE,	/*10 */
-							 _XA_NET_WM_STATE_MODAL, _XA_NET_WM_STATE_STICKY, _XA_NET_WM_STATE_MAXIMIZED_VERT, _XA_NET_WM_STATE_MAXIMIZED_HORZ, _XA_NET_WM_STATE_SHADED, _XA_NET_WM_STATE_SKIP_TASKBAR, _XA_NET_WM_PID, _XA_NET_WM_PING);	/*19 */
+							 _XA_NET_WM_NAME, 					_XA_NET_WM_DESKTOP, 			_XA_NET_WM_WINDOW_TYPE, 
+							 _XA_NET_WM_WINDOW_TYPE_DESKTOP, 	_XA_NET_WM_WINDOW_TYPE_DOCK, 	_XA_NET_WM_WINDOW_TYPE_TOOLBAR, 
+							 _XA_NET_WM_WINDOW_TYPE_MENU, 		_XA_NET_WM_WINDOW_TYPE_DIALOG, 	_XA_NET_WM_WINDOW_TYPE_NORMAL, 
+							 _XA_NET_WM_STATE,					_XA_NET_WM_STATE_MODAL,    		_XA_NET_WM_STATE_STICKY,
+							 _XA_NET_WM_STATE_MAXIMIZED_VERT, 	_XA_NET_WM_STATE_MAXIMIZED_HORZ,_XA_NET_WM_STATE_SHADED, 		
+							 _XA_NET_WM_STATE_SKIP_TASKBAR, 	_XA_NET_WM_PID, 				_XA_NET_WM_PING, 				
+							 _XA_NET_WM_ICON);	/*19 */
 
 }
 
@@ -520,17 +565,15 @@ read_as_background (ASWMProps * wmprops, Bool deleted)
 
 	if (wmprops)
 	{
-		if (wmprops->as_backgrounds_data)
-		{
-			free (wmprops->as_backgrounds_data);
-			wmprops->as_backgrounds_data = NULL;
-		}
-		wmprops->as_backgrounds_size = 0;
+        CARD32 pmap_id = None;
+
 		if (deleted)
 			return False;
-		res = read_as_property (wmprops->selection_window, _AS_BACKGROUND,
-								&(wmprops->as_backgrounds_size),
-								&(wmprops->as_backgrounds_version), &(wmprops->as_backgrounds_data));
+		if (read_32bit_property (wmprops->selection_window, _AS_BACKGROUND, &pmap_id))
+		{
+			wmprops->as_root_pixmap = (Pixmap) pmap_id;
+			return True;
+		}
 	}
 	return res;
 }
@@ -598,6 +641,47 @@ read_as_virtual_root (ASWMProps * wmprops, Bool deleted)
 	return False;
 }
 
+Bool
+read_as_service_window (ASWMProps * wmprops, Bool deleted)
+{
+	if (wmprops)
+	{
+        CARD32        swin_id = None;
+
+		if (deleted)
+			return False;
+		if (read_32bit_property (wmprops->selection_window, _AS_SERVICE_WINDOW, &swin_id))
+		{
+			wmprops->as_service_window = (Window) swin_id;
+			return True;
+		}
+	}
+	return False;
+}
+
+Bool
+read_as_tbar_props (ASWMProps * wmprops, Bool deleted)
+{
+	Bool          res = False;
+
+	if (wmprops)
+	{
+		if (wmprops->as_tbar_props_data)
+		{
+			free (wmprops->as_tbar_props_data);
+			wmprops->as_tbar_props_data = NULL;
+		}
+		wmprops->as_tbar_props_size = 0;
+		if (deleted)
+			return False;
+		res = read_as_property (wmprops->selection_window, _AS_TBAR_PROPS,
+								&(wmprops->as_tbar_props_size), &(wmprops->as_tbar_props_version), &(wmprops->as_tbar_props_data));
+	}
+	return res;
+}
+
+
+
 /******************** Property management functions: **************************/
 
 void
@@ -656,6 +740,9 @@ prop_description_struct WMPropsDescriptions_volitile[] = {
 	{&_AS_VISUAL, read_as_visual, WMC_ASVisual, 0},
 	{&_AS_MODULE_SOCKET, read_as_module_socket, WMC_ASModule, 0},
 	{&_AS_VIRTUAL_ROOT, read_as_virtual_root, WMC_ASVirtualRoot, 0},
+	{&_AS_SERVICE_WINDOW, read_as_service_window, WMC_ASServiceWindow, 0},
+	{&_AS_TBAR_PROPS, read_as_tbar_props, WMC_ASTBarProps, 0},
+	
 	{NULL, NULL, 0, 0}
 };
 
@@ -809,7 +896,10 @@ setup_wmprops (ScreenInfo * scr, Bool manager, ASFlagType what, ASWMProps * reus
 
 		read_root_wmprops (wmprops, manager);
 		if (!manager)
+		{	
 			read_volitile_wmprops (wmprops);
+			scr->ServiceWin = wmprops->as_service_window ;
+		}
 	}
 	return wmprops;
 }
@@ -845,14 +935,14 @@ destroy_wmprops (ASWMProps * wmprops, Bool reusable)
 			free (wmprops->stacking_order);
 		if (wmprops->as_styles_data)
 			free (wmprops->as_styles_data);
-		if (wmprops->as_backgrounds_data)
-			free (wmprops->as_backgrounds_data);
 		if (wmprops->as_visual_data)
 			free (wmprops->as_visual_data);
 		if (wmprops->as_socket_filename)
 			free (wmprops->as_socket_filename);
 		if (wmprops->as_desk_numbers)
 			free (wmprops->as_desk_numbers);
+		if (wmprops->as_tbar_props_data)
+			free (wmprops->as_tbar_props_data);
 
         /* we are being paranoid : */
         memset (wmprops, 0x00, sizeof (ASWMProps));
@@ -882,7 +972,7 @@ set_prop_updated (ASHashTable * handlers, Atom prop)
 /*    prop_description_struct *descr ;
     ASHashableValue  hprop = (ASHashableValue)((unsigned long)prop);
     if( get_hash_item( handlers, hprop, (void**)&descr ) == ASH_Success )
-        descr->update_time = Scr.last_Timestamp ;
+        descr->update_time = ASDefaultScr->last_Timestamp ;
  */
 }
 
@@ -930,31 +1020,45 @@ set_as_style (ASWMProps * wmprops, CARD32 size, CARD32 version, CARD32 *data)
 }
 
 void
-set_as_backgrounds (ASWMProps * wmprops, CARD32 size, CARD32 version, CARD32 *data)
+set_as_background (ASWMProps * wmprops, Pixmap new_pmap)
 {
 	if (wmprops)
 	{
 		if (wmprops->selection_window == None)
 			return;
-		if (data == NULL || size == 0)
-			XDeleteProperty (dpy, wmprops->selection_window, _AS_BACKGROUND);
-		else
-			set_as_property (wmprops->selection_window, _AS_BACKGROUND, data, size, version);
+        
+		set_32bit_property (wmprops->selection_window, _AS_BACKGROUND, XA_PIXMAP, new_pmap);
+		XFlush (dpy);
+        wmprops->as_root_pixmap = new_pmap;
+	}
+}
 
-		if (wmprops->as_backgrounds_data && (size > wmprops->as_backgrounds_size || data == NULL))
-		{
-			free (wmprops->as_backgrounds_data);
-			wmprops->as_backgrounds_data = NULL;
-		}
-		if (data)
-		{
-			if (wmprops->as_backgrounds_data == NULL)
-				wmprops->as_backgrounds_data = safemalloc (size);
-			memcpy (wmprops->as_backgrounds_data, data, size);
-		}
+void
+set_xrootpmap_id (ASWMProps * wmprops, Pixmap new_pmap)
+{
+	if (wmprops)
+	{
+        set_32bit_property (wmprops->scr->Root, _XROOTPMAP_ID, XA_PIXMAP, new_pmap);
+		XFlush (dpy);
+        wmprops->root_pixmap = new_pmap;
+	}
+}
 
-		wmprops->as_backgrounds_size = size;
-		wmprops->as_backgrounds_version = version;
+void
+validate_rootpmap_props(ASWMProps *wmprops)
+{
+	if( wmprops )
+	{
+		if( wmprops->root_pixmap )
+		{	
+			if( !validate_drawable( wmprops->root_pixmap, NULL, NULL ) )
+	    		set_xrootpmap_id (wmprops, None );
+		}
+		if( wmprops->as_root_pixmap ) 
+		{	
+			if( !validate_drawable( wmprops->as_root_pixmap, NULL, NULL ) )
+	    		set_as_background (wmprops, None );
+		}
 	}
 }
 
@@ -1126,6 +1230,18 @@ set_desktop_geometry_prop (ASWMProps * wmprops, CARD32 width, CARD32 height)
 	return False;
 }
 
+void
+set_service_window_prop (ASWMProps * wmprops, Window service_win)
+{
+	if (wmprops)
+	{
+        if ( wmprops->as_service_window == service_win )
+			return ;
+        wmprops->as_service_window = service_win ;
+        set_32bit_property (wmprops->selection_window, _AS_SERVICE_WINDOW, XA_WINDOW, service_win);
+    }
+}
+
 
 void
 flush_wmprop_data (ASWMProps * wmprops, ASFlagType what)
@@ -1165,16 +1281,6 @@ flush_wmprop_data (ASWMProps * wmprops, ASFlagType what)
 	}
 }
 
-void
-set_xrootpmap_id (ASWMProps * wmprops, Pixmap new_pmap)
-{
-	if (wmprops)
-	{
-        set_32bit_property (wmprops->scr->Root, _XROOTPMAP_ID, XA_PIXMAP, new_pmap);
-		XFlush (dpy);
-        wmprops->root_pixmap = new_pmap;
-	}
-}
 
 void
 set_clients_list (ASWMProps * wmprops, Window *list, int nclients)
@@ -1227,6 +1333,94 @@ set_active_window_prop (ASWMProps * wmprops, Window active)
 	}
 }
 
+ASTBarProps *
+get_astbar_props( ASWMProps *wmprops ) 
+{
+	ASTBarProps *tbar_props = NULL;
+	if( wmprops && wmprops->as_tbar_props_size > 0 && wmprops->as_tbar_props_data != NULL ) 
+	{
+		CARD32 *prop = wmprops->as_tbar_props_data ;
+		int i, nbuttons ; 
+		tbar_props = safecalloc( 1, sizeof(ASTBarProps));
+		tbar_props->align = prop[0] ; 
+		tbar_props->bevel = prop[1] ; 
+		tbar_props->title_h_spacing = prop[2] ;
+		tbar_props->title_v_spacing = prop[3] ;
+		tbar_props->buttons_h_border = prop[4] ; 
+		tbar_props->buttons_v_border = prop[5] ; 
+		tbar_props->buttons_spacing = prop[6] ; 
+		tbar_props->buttons_num = prop[7] ; 
+		nbuttons = tbar_props->buttons_num;
+		if( nbuttons > 10 || (nbuttons*4+8)*sizeof(CARD32) > wmprops->as_tbar_props_size)
+		{
+			tbar_props->buttons_num = 0 ;
+		}else
+		{		 
+			tbar_props->buttons = safemalloc( nbuttons*sizeof(struct ASButtonPropElem));
+			for( i = 0 ; i < nbuttons ; ++i ) 
+			{
+				tbar_props->buttons[i].kind = prop[8+i*3];
+				tbar_props->buttons[i].pmap = prop[8+i*3+1];
+				tbar_props->buttons[i].mask = prop[8+i*3+2];
+				tbar_props->buttons[i].alpha = prop[8+i*3+3];
+			}	 
+		}
+	}	 
+	
+	return tbar_props;	   
+	
+}
+
+void
+set_astbar_props( ASWMProps *wmprops, ASTBarProps *tbar_props ) 
+{
+	CARD32 *prop = NULL;
+	int size = 0;
+	CARD32 version = (1 << 8) + 1 ; /* version 1.1 */
+
+	if (wmprops == NULL || wmprops->selection_window == None )
+			return;
+
+	if( tbar_props ) 
+	{
+		int i, nbuttons = tbar_props->buttons_num;
+		size = sizeof (CARD32) * (nbuttons*4 + 8);
+		prop = safecalloc ( 1, size );
+		prop[0] = tbar_props->align ; 
+		prop[1] = tbar_props->bevel ; 
+		prop[2] = tbar_props->title_h_spacing ;
+		prop[3] = tbar_props->title_v_spacing ;
+		prop[4] = tbar_props->buttons_h_border ; 
+		prop[5] = tbar_props->buttons_v_border ; 
+		prop[6] = tbar_props->buttons_spacing ; 
+		prop[7] = tbar_props->buttons_num ; 
+	
+
+		for( i = 0 ; i < nbuttons ; ++i ) 
+		{
+			prop[8+i*3] = tbar_props->buttons[i].kind ;
+			prop[8+i*3+1] = tbar_props->buttons[i].pmap ;
+			prop[8+i*3+2] = tbar_props->buttons[i].mask ;
+			prop[8+i*3+3] = tbar_props->buttons[i].alpha ;
+		}	 
+	}
+
+	if (prop == NULL || size == 0)
+		XDeleteProperty (dpy, wmprops->selection_window, _AS_TBAR_PROPS);
+	else
+		set_as_property (wmprops->selection_window, _AS_TBAR_PROPS, prop, size, version);
+
+	if (wmprops->as_tbar_props_data )
+		free (wmprops->as_tbar_props_data);
+	
+	wmprops->as_tbar_props_data  = prop ;
+
+	wmprops->as_tbar_props_size = size;
+	wmprops->as_tbar_props_version = version;
+}
+
+
+
 /********************************************************************************/
 /* We need to handle Property Notify Events : 						            */
 /********************************************************************************/
