@@ -47,6 +47,7 @@
 #include "../../libAfterStep/balloon.h"
 #include "../../libAfterStep/event.h"
 #include "../../libAfterStep/session.h"
+#include "../../libAfterStep/desktop_category.h"
 
 #include "../../libAfterConf/afterconf.h"
 
@@ -78,13 +79,11 @@ typedef struct {
 ASIdentState IdentState = {};
 
 IdentConfig *Config = NULL ;
-struct ASDatabase    *Database = NULL;
 /**********************************************************************/
 Window get_target_window();
 void GetBaseOptions (const char *filename);
 void GetOptions (const char *filename);
 void CheckConfigSanity();
-void ParseDatabase ();
 
 void HandleEvents();
 void DispatchEvent (ASEvent * event);
@@ -119,7 +118,8 @@ main( int argc, char **argv )
 	LoadColorScheme();
 	LoadConfig ("ident", GetOptions);
     CheckConfigSanity();
-	ParseDatabase ();
+	ReloadASDatabase();
+	ReloadCategories(True);
 
 	if (MyArgs.src_window == 0)
 		MyArgs.src_window = get_target_window();
@@ -216,30 +216,6 @@ GetOptions (const char *filename)
     SHOW_TIME("Config parsing",option_time);
 }
 
-void
-ParseDatabase ()
-{
-    struct name_list *list = NULL ;
-	char *file = make_session_file(Session, DATABASE_FILE, False );
-	
-	/* memory management for parsing buffer */
-	if (file == NULL)
-        return ;
-
-    list = ParseDatabaseOptions (file, "afterstep");
-    if( list )
-    {
-        Database = build_asdb( list );
-        if( is_output_level_under_threshold( OUTPUT_LEVEL_DATABASE ) )
-            print_asdb( NULL, NULL, Database );
-        while (list != NULL)
-            delete_name_list (&(list));
-    }else
-        show_progress( "no database records loaded." );
-    /* XResources : */
-    load_user_database();
-}
-
 /****************************************************************************/
 /* PROCESSING OF AFTERSTEP MESSAGES :                                       */
 /****************************************************************************/
@@ -396,6 +372,7 @@ make_ident_window( int width, int height)
     extwm_hints.flags = EXTWM_PID|EXTWM_TypeASModule ;
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
+	set_client_cmd (w);
 
 	/* showing window to let user see that we are doing something */
 	XMapRaised (dpy, w);
@@ -454,7 +431,8 @@ struct
 	char *name_set ;	  
 	char *name_unset ;	  
 } DatabaseFlagsList[] = 
-{	{STYLE_STICKY, 				"Sticky", 				"Slippery"},
+{	{STYLE_FULLSCREEN,			"Fullscreen",			"NoFullscreen"},
+	{STYLE_STICKY, 				"Sticky", 				"Slippery"},
 	{STYLE_TITLE, 				"Title", 				"NoTitle"},
 	{STYLE_CIRCULATE, 			"CirculateHit", 		"CirculateSkip"},
 	{STYLE_WINLIST, 			"WindowListHit", 		"WindowListSkip"},
@@ -533,16 +511,23 @@ fill_window_data()
 	do{ if( get_flags( flags, WIN_##type##_##flg)){ if( buf[0] != '\0' ) strcat( buf, ", "); strcat( buf, #flg );}}while(0)
 
 	buf[0] = '\0' ;
+	if( (wd->state_flags & (AS_MaximizedX|AS_MaximizedY)) == (AS_MaximizedX|AS_MaximizedY) )
+	{
+		strcpy( buf, "Maximized"); 
+	}else
+	{
+		SHOW_FLAG(wd->state_flags,MaximizedX);
+		SHOW_FLAG(wd->state_flags,MaximizedY);    
+	}
 	SHOW_FLAG(wd->state_flags,Iconic);
-	SHOW_FLAG(wd->state_flags,MaximizedX);
-	SHOW_FLAG(wd->state_flags,MaximizedY);    
+	SHOW_FLAG(wd->state_flags,Fullscreen);    
 	SHOW_FLAG(wd->state_flags,Sticky);    
 	SHOW_FLAG(wd->state_flags,Shaded);        
 	SHOW_FLAG(wd->state_flags,Withdrawn);
 	SHOW_FLAG(wd->state_flags,Dead);     
 	SHOW_FLAG(wd->state_flags,Mapped);
 	SHOW_FLAG(wd->state_flags,IconMapped);
-	SHOW_FLAG(wd->state_flags,Visible);
+	SHOW_FLAG(wd->state_flags,Hidden);
 	SHOW_FLAG(wd->state_flags,Shaped);
 	SHOW_FLAG(wd->state_flags,ShapedDecor);
 	SHOW_FLAG(wd->state_flags,ShapedIcon);
@@ -798,6 +783,20 @@ fill_window_data()
 			add_property("Style:", buf, AS_Text_ASCII, True);
 		}	 
 	}	 
+	{
+		ASDesktopEntry *de = fetch_desktop_entry( CombinedCategories, names[2]);
+		if( de == NULL ) 
+			de = fetch_desktop_entry( CombinedCategories, names[3]);				  
+		if( de ) 
+		{	
+			char *categories = make_desktop_entry_categories( de );
+			add_property("Categories:", categories?categories:"<none>", AS_Text_ASCII, False);
+			destroy_string( &categories ) ;
+			add_property(".desktop Entry Name:", de->Name, AS_Text_ASCII, False);
+			add_property(".desktop Entry Comment:", de->Comment, AS_Text_ASCII, True);
+			add_property(".desktop Entry Icon:", de->fulliconname, AS_Text_ASCII, True);
+		}
+	}
 }
 
 void 
