@@ -84,6 +84,8 @@ typedef struct {
 	Bool postpone_display ;
 	time_t last_message_time ;
 
+	ASTBarProps *tbar_props ;
+
 }ASWinListState ;
 
 ASWinListState WinListState = { 0, NULL, None, NULL, NULL };
@@ -102,7 +104,7 @@ WinListConfig *Config = NULL ;
 
 /**********************************************************************/
 
-void CheckConfigSanity();
+void SetWinListLook();
 void GetBaseOptions (const char *filename);
 void GetOptions (const char *filename);
 void HandleEvents();
@@ -125,15 +127,17 @@ main( int argc, char **argv )
     /* Save our program name - for error messages */
 	set_DeadPipe_handler(DeadPipe);
     InitMyApp (CLASS_WINLIST, argc, argv, NULL, NULL, 0 );
+	LinkAfterStepConfig();
 
     set_signal_handler( SIGSEGV );
 
-    ConnectX( ASDefaultScr, PropertyChangeMask|EnterWindowMask );
-    ConnectAfterStep (M_FOCUS_CHANGE |
+    ConnectX( ASDefaultScr, EnterWindowMask );
+    if( ConnectAfterStep (M_FOCUS_CHANGE |
                       M_DESTROY_WINDOW |
                       WINDOW_CONFIG_MASK |
                       WINDOW_NAME_MASK |
-                      M_END_WINDOWLIST);
+                      M_END_WINDOWLIST, 0) < 0 ) 
+		exit(1);               /* no AfterStep */
     balloon_init (False);
     Config = CreateWinListConfig ();
 
@@ -143,7 +147,11 @@ main( int argc, char **argv )
     LoadBaseConfig ( GetBaseOptions);
 	LoadColorScheme();
 	LoadConfig ("winlist", GetOptions);
-    CheckConfigSanity();
+	CheckWinListConfigSanity(Config, &(MyArgs.geometry), MyArgs.gravity);
+	ReloadASDatabase();
+	ReloadCategories(True);
+    
+	SetWinListLook();
 
     WinListState.main_window = make_winlist_window();
     WinListState.main_canvas = create_ascanvas( WinListState.main_window );
@@ -208,51 +216,33 @@ DeadPipe (int nonsense)
 }
 
 void
-CheckConfigSanity()
+retrieve_winlist_astbar_props()
+{
+	destroy_astbar_props( &(WinListState.tbar_props) );
+	
+	WinListState.tbar_props = get_astbar_props(Scr.wmprops );
+}	 
+
+
+void
+SetWinListLook()
 {
     int i ;
     char *default_winlist_style = safemalloc( 1+strlen(MyName)+1);
 	default_winlist_style[0] = '*' ;
 	strcpy( &(default_winlist_style[1]), MyName );
 
-    if( Config == NULL )
-        Config = CreateWinListConfig ();
-
-    if( Config->max_rows > MAX_WINLIST_WINDOW_COUNT || Config->max_rows == 0  )
-        Config->max_rows = MAX_WINLIST_WINDOW_COUNT;
-
-    if( Config->max_columns > MAX_WINLIST_WINDOW_COUNT || Config->max_columns == 0  )
-        Config->max_columns = MAX_WINLIST_WINDOW_COUNT;
-
-    Config->gravity = NorthWestGravity ;
-    if( get_flags(Config->geometry.flags, XNegative) )
-        Config->gravity = get_flags(Config->geometry.flags, YNegative)? SouthEastGravity:NorthEastGravity;
-    else if( get_flags(Config->geometry.flags, YNegative) )
-        Config->gravity = SouthWestGravity;
-
-    Config->anchor_x = get_flags( Config->geometry.flags, XValue )?Config->geometry.x:0;
-    if( get_flags(Config->geometry.flags, XNegative) )
-        Config->anchor_x += Scr.MyDisplayWidth ;
-
-    Config->anchor_y = get_flags( Config->geometry.flags, YValue )?Config->geometry.y:0;
-    if( get_flags(Config->geometry.flags, YNegative) )
-        Config->anchor_y += Scr.MyDisplayHeight ;
-
+	retrieve_winlist_astbar_props();
     mystyle_get_property (Scr.wmprops);
+    for( i = 0 ; i < BACK_STYLES ; ++i )
+		Scr.Look.MSWindow[i] = NULL ;
 
-    /* we better not do this to introduce ppl to new concepts in WinList : */
-#if 0
-    if( Config->focused_style == NULL )
-        Config->focused_style = mystrdup( default_winlist_style );
-    if( Config->unfocused_style == NULL )
-        Config->unfocused_style = mystrdup( default_winlist_style );
-    if( Config->sticky_style == NULL )
-        Config->sticky_style = mystrdup( default_winlist_style );
-#endif
-
-    Scr.Look.MSWindow[BACK_UNFOCUSED] = mystyle_find( Config->unfocused_style );
-    Scr.Look.MSWindow[BACK_FOCUSED] = mystyle_find( Config->focused_style );
-    Scr.Look.MSWindow[BACK_STICKY] = mystyle_find( Config->sticky_style );
+    Scr.Look.MSWindow[BACK_UNFOCUSED] = mystyle_find( Config->UnfocusedStyle );
+	LOCAL_DEBUG_OUT( "Configured MyStyle %d \"%s\" is %p", BACK_UNFOCUSED, Config->UnfocusedStyle?Config->UnfocusedStyle:"(null)", Scr.Look.MSWindow[BACK_UNFOCUSED] );
+    Scr.Look.MSWindow[BACK_FOCUSED] = mystyle_find( Config->FocusedStyle );
+	LOCAL_DEBUG_OUT( "Configured MyStyle %d \"%s\" is %p", BACK_FOCUSED, Config->FocusedStyle?Config->FocusedStyle:"(null)", Scr.Look.MSWindow[BACK_FOCUSED] );
+    Scr.Look.MSWindow[BACK_STICKY] = mystyle_find( Config->StickyStyle );
+	LOCAL_DEBUG_OUT( "Configured MyStyle %d \"%s\" is %p", BACK_STICKY, Config->StickyStyle?Config->StickyStyle:"(null)", Scr.Look.MSWindow[BACK_STICKY] );
 
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
@@ -261,16 +251,15 @@ CheckConfigSanity()
             Scr.Look.MSWindow[i] = mystyle_find( default_window_style_name[i] );
         if( Scr.Look.MSWindow[i] == NULL )
             Scr.Look.MSWindow[i] = mystyle_find_or_default( default_winlist_style );
+		LOCAL_DEBUG_OUT( "MyStyle %d is \"%s\"", i, Scr.Look.MSWindow[i]?Scr.Look.MSWindow[i]->name:"none" );
     }
     free( default_winlist_style );
-
+	
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
     PrintWinListConfig (Config);
     Print_balloonConfig ( Config->balloon_conf );
 #endif
-    balloon_config2look( &(Scr.Look), Config->balloon_conf );
-    LOCAL_DEBUG_OUT( "balloon mystyle = %p (\"%s\")", Scr.Look.balloon_look->style,
-                    Scr.Look.balloon_look->style?Scr.Look.balloon_look->style->name:"none" );
+    balloon_config2look( &(Scr.Look), Config->balloon_conf, "*WinListBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
 
 }
@@ -279,13 +268,12 @@ CheckConfigSanity()
 void
 GetBaseOptions (const char *filename)
 {
-	ReloadASEnvironment( NULL, NULL, NULL, False );
+	ReloadASEnvironment( NULL, NULL, NULL, False, True );
 }
 
 void
 GetOptions (const char *filename)
 {
-    int i ;
     START_TIME(option_time);
     WinListConfig *config = ParseWinListOptions( filename, MyName );
 
@@ -293,82 +281,14 @@ GetOptions (const char *filename)
     PrintWinListConfig (config);
 #endif
     /* Need to merge new config with what we have already :*/
-    /* now lets check the config sanity : */
-    /* mixing set and default flags : */
-    Config->flags = (config->flags&config->set_flags)|(Config->flags & (~config->set_flags));
-    Config->set_flags |= config->set_flags;
-
-    Config->gravity = NorthWestGravity ;
-    if( get_flags(config->set_flags, WINLIST_Geometry) )
-        merge_geometry(&(config->geometry), &(Config->geometry) );
-
-    if( get_flags(config->set_flags, WINLIST_MinSize) )
-    {
-        Config->min_width = config->min_width;
-        Config->min_height = config->min_height;
-    }
-
-    if( get_flags(config->set_flags, WINLIST_MaxSize) )
-    {
-        Config->max_width = config->max_width;
-        Config->max_height = config->max_height;
-    }
-    if( get_flags(config->set_flags, WINLIST_MaxRows) )
-        Config->max_rows = config->max_rows;
-    if( get_flags(config->set_flags, WINLIST_MaxColumns) )
-        Config->max_columns = config->max_columns;
-    if( get_flags(config->set_flags, WINLIST_MaxColWidth) )
-        Config->max_col_width = config->max_col_width;
-    if( get_flags(config->set_flags, WINLIST_MinColWidth) )
-        Config->min_col_width = config->min_col_width;
-
-    if( config->unfocused_style )
-        set_string_value( &(Config->unfocused_style), mystrdup(config->unfocused_style), NULL, 0 );
-    if( config->focused_style )
-        set_string_value( &(Config->focused_style), mystrdup(config->focused_style), NULL, 0 );
-    if( config->sticky_style )
-        set_string_value( &(Config->sticky_style), mystrdup(config->sticky_style), NULL, 0 );
-
-    if( get_flags(config->set_flags, WINLIST_UseName) )
-        Config->show_name_type = config->show_name_type;
-    if( get_flags(config->set_flags, WINLIST_Align) )
-        Config->name_aligment = config->name_aligment;
-    if( get_flags(config->set_flags, WINLIST_FBevel) )
-        Config->fbevel = config->fbevel;
-    if( get_flags(config->set_flags, WINLIST_UBevel) )
-        Config->ubevel = config->ubevel;
-    if( get_flags(config->set_flags, WINLIST_SBevel) )
-        Config->sbevel = config->sbevel;
-
-    if( get_flags(config->set_flags, WINLIST_FCM) )
-        Config->fcm = config->fcm;
-    if( get_flags(config->set_flags, WINLIST_UCM) )
-        Config->ucm = config->ucm;
-    if( get_flags(config->set_flags, WINLIST_SCM) )
-        Config->scm = config->scm;
-
-    if( get_flags(config->set_flags, WINLIST_H_SPACING) )
-        Config->h_spacing = config->h_spacing;
-    if( get_flags(config->set_flags, WINLIST_V_SPACING) )
-        Config->v_spacing = config->v_spacing;
-
-    for( i = 0 ; i < MAX_MOUSE_BUTTONS ; ++i )
-        if( config->mouse_actions[i] )
-        {
-            destroy_string_list( Config->mouse_actions[i] );
-            Config->mouse_actions[i] = config->mouse_actions[i];
-        }
-
-    if( Config->balloon_conf )
-        Destroy_balloonConfig( Config->balloon_conf );
-    Config->balloon_conf = config->balloon_conf ;
-    config->balloon_conf = NULL ;
+	MergeWinListOptions( Config, config );
 
     if (config->style_defs)
         ProcessMyStyleDefinitions (&(config->style_defs));
     SHOW_TIME("Config parsing",option_time);
 }
 
+
 /****************************************************************************/
 /* PROCESSING OF AFTERSTEP MESSAGES :                                       */
 /****************************************************************************/
@@ -394,7 +314,7 @@ process_message (send_data_type type, send_data_type *body)
 		if( res == WP_DataCreated )
         {
             if( WinListState.windows_num < MAX_WINLIST_WINDOW_COUNT &&
-                WinListState.windows_num < Config->max_rows*Config->max_columns )
+                WinListState.windows_num < Config->MaxRows*Config->MaxColumns )
             {
                 if( !get_flags( Config->flags, ASWL_UseSkipList ) ||
                     !get_flags( wd->flags, AS_SkipWinList ) )
@@ -412,13 +332,18 @@ void
 DispatchEvent (ASEvent * event)
 {
     ASWindowData *pointer_wd = NULL ;
-
+	static Bool root_pointer_moved = True ;
     SHOW_EVENT_TRACE(event);
-
+	
     if( (event->eclass & ASE_POINTER_EVENTS) != 0 )
     {
         int i  = find_button_by_position( event->x.xmotion.x_root - (WinListState.main_canvas->root_x+(int)WinListState.main_canvas->bw),
                                           event->x.xmotion.y_root - (WinListState.main_canvas->root_y+(int)WinListState.main_canvas->bw) );
+		if( is_balloon_click( &(event->x) ) )
+		{
+			withdraw_balloon(NULL);
+			return;
+		}
         if( i < WinListState.windows_num )
             pointer_wd = WinListState.window_order[i] ;
     }
@@ -457,8 +382,13 @@ DispatchEvent (ASEvent * event)
 			}
         case LeaveNotify :
         case MotionNotify :
+			if( event->x.type == MotionNotify ) 
+				root_pointer_moved = True ; 
             if( pointer_wd )
-                on_astbar_pointer_action( pointer_wd->bar, 0, (event->x.type == LeaveNotify) );
+			{	
+                on_astbar_pointer_action( pointer_wd->bar, 0, (event->x.type == LeaveNotify), root_pointer_moved );
+				root_pointer_moved = False ; 
+			}
             break ;
 	    case ClientMessage:
             if ((event->x.xclient.format == 32) &&
@@ -469,35 +399,56 @@ DispatchEvent (ASEvent * event)
 	        break;
 	    case PropertyNotify:
 			LOCAL_DEBUG_OUT( "property %s(%lX), _XROOTPMAP_ID = %lX, event->w = %lX, root = %lX", XGetAtomName(dpy, event->x.xproperty.atom), event->x.xproperty.atom, _XROOTPMAP_ID, event->w, Scr.Root );
-            if( event->x.xproperty.atom == _XROOTPMAP_ID && event->w == Scr.Root )
-            {
-                int i ;
-                LOCAL_DEBUG_OUT( "root background updated!%s","");
-                safe_asimage_destroy( Scr.RootImage );
-                Scr.RootImage = NULL ;
-                for( i = 0 ; i < WinListState.windows_num ; ++i )
-                    if( update_astbar_transparency( WinListState.window_order[i]->bar, WinListState.main_canvas, True ) )
-						render_astbar( WinListState.window_order[i]->bar, WinListState.main_canvas );
-		        if( is_canvas_dirty( WinListState.main_canvas ) )
-				{
-					LOCAL_DEBUG_OUT( "update main canvas%s","");
-            		update_canvas_display( WinListState.main_canvas );
-				}
-            }else if( event->x.xproperty.atom == _AS_STYLE )
+			if( event->w == Scr.Root || event->w == Scr.wmprops->selection_window ) 
 			{
-                int i ;
-				LOCAL_DEBUG_OUT( "AS Styles updated!%s","");
 				handle_wmprop_event (Scr.wmprops, &(event->x));
-				mystyle_list_destroy_all(&(Scr.Look.styles_list));
-				LoadColorScheme();
-				CheckConfigSanity();
-				/* now we need to update everything */
-				update_winlist_styles();
-				rearrange_winlist_window( False );
-                for( i = 0 ; i < WinListState.windows_num ; ++i )
-					refresh_winlist_button( WinListState.window_order[i]->bar, WinListState.window_order[i], False );
+            	if( event->x.xproperty.atom == _AS_BACKGROUND )
+            	{
+                	int i ;
+                	LOCAL_DEBUG_OUT( "root background updated!%s","");
+                	safe_asimage_destroy( Scr.RootImage );
+                	Scr.RootImage = NULL ;
+                	for( i = 0 ; i < WinListState.windows_num ; ++i )
+                    	if( update_astbar_transparency( WinListState.window_order[i]->bar, WinListState.main_canvas, True ) )
+							render_astbar( WinListState.window_order[i]->bar, WinListState.main_canvas );
+		        	if( is_canvas_dirty( WinListState.main_canvas ) )
+					{
+						LOCAL_DEBUG_OUT( "update main canvas%s","");
+            			update_canvas_display( WinListState.main_canvas );
+					}
+            	}else if( event->x.xproperty.atom == _AS_STYLE )
+				{
+                	int i ;
+					LOCAL_DEBUG_OUT( "AS Styles updated!%s","");
+					mystyle_list_destroy_all(&(Scr.Look.styles_list));
+					LoadColorScheme();
+					SetWinListLook();
+					/* now we need to update everything */
+					update_winlist_styles();
+					rearrange_winlist_window( False );
+                	for( i = 0 ; i < WinListState.windows_num ; ++i )
+						refresh_winlist_button( WinListState.window_order[i]->bar, WinListState.window_order[i], False );
+				}else if( event->x.xproperty.atom == _AS_TBAR_PROPS )
+				{
+					int i ;
+		 			retrieve_winlist_astbar_props();		
+					update_winlist_styles();
+					rearrange_winlist_window( False );
+                	for( i = 0 ; i < WinListState.windows_num ; ++i )
+						refresh_winlist_button( WinListState.window_order[i]->bar, WinListState.window_order[i], False );
+            	}
+			}else if( event->x.xproperty.atom == _XA_NET_WM_ICON )
+			{                  /* Maybe name change on the client !!! */
+				/*handle_tab_name_change( event->w ); */
 			}
 			break;
+		default:
+#ifdef XSHMIMAGE
+			LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : completion_type = %d, event->type = %d ", Scr.ShmCompletionEventType, event->x.type );
+			if( event->x.type == Scr.ShmCompletionEventType )
+				handle_ShmCompletion( event );
+#endif /* SHAPE */
+			break;
     }
 }
 
@@ -511,8 +462,8 @@ make_winlist_window()
 	XSizeHints    shints;
 	ExtendedWMHints extwm_hints ;
 	int x, y ;
-    unsigned int width = max(Config->min_width,1);
-    unsigned int height = max(Config->min_height,1);
+    unsigned int width = max(Config->MinSize.width,1);
+    unsigned int height = max(Config->MinSize.height,1);
 
 	switch( Config->gravity )
 	{
@@ -540,8 +491,8 @@ make_winlist_window()
 
 	shints.flags = USPosition|USSize|PMinSize|PMaxSize|PBaseSize|PWinGravity;
 	shints.min_width = shints.min_height = 4;
-	shints.max_width = (Config->max_width>0)?Config->max_width:Scr.MyDisplayWidth;
-	shints.max_height = (Config->max_height>0)?Config->max_height:Scr.MyDisplayHeight;
+	shints.max_width = (Config->MaxSize.width>0)?Config->MaxSize.width:Scr.MyDisplayWidth;
+	shints.max_height = (Config->MaxSize.height>0)?Config->MaxSize.height:Scr.MyDisplayHeight;
 	shints.base_width = shints.base_height = 4;
 	shints.win_gravity = Config->gravity ;
 
@@ -549,6 +500,7 @@ make_winlist_window()
 	extwm_hints.flags = EXTWM_PID|EXTWM_StateSkipTaskbar|EXTWM_StateSkipPager|EXTWM_TypeDock|EXTWM_TypeASModule ;
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
+	set_client_cmd (w);
 
 	/* showing window to let user see that we are doing something */
 	XMapRaised (dpy, w);
@@ -598,11 +550,11 @@ Bool
 rearrange_winlist_window( Bool dont_resize_main_canvas )
 {
     int i ;
-    unsigned int allowed_max_width = (Config->max_width==0)?Scr.MyDisplayWidth:Config->max_width ;
-	unsigned int allowed_max_height = (Config->max_height==0)?Scr.MyDisplayHeight:Config->max_height ;
-    unsigned int allowed_min_width = Config->min_width ;
-    unsigned int allowed_min_height = Config->min_height;
-    unsigned int max_col_width = (Config->max_col_width==0)?Scr.MyDisplayWidth:Config->max_col_width ;
+    unsigned int allowed_max_width = (Config->MaxSize.width==0)?Scr.MyDisplayWidth:Config->MaxSize.width ;
+	unsigned int allowed_max_height = (Config->MaxSize.height==0)?Scr.MyDisplayHeight:Config->MaxSize.height ;
+    unsigned int allowed_min_width = Config->MinSize.width ;
+    unsigned int allowed_min_height = Config->MinSize.height;
+    unsigned int max_col_width = (Config->MaxColWidth==0)?Scr.MyDisplayWidth:Config->MaxColWidth ;
     unsigned int max_item_height = 0;
     unsigned int max_rows = 0 ;
     int row = 0, col = 0;
@@ -695,8 +647,8 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
 	LOCAL_DEBUG_OUT( "calculated max_item_height = %d", max_item_height );
 
     max_rows = (allowed_max_height + max_item_height - 1 ) / max_item_height ;
-    if( max_rows > Config->max_rows )
-        max_rows = Config->max_rows ;
+    if( max_rows > Config->MaxRows )
+        max_rows = Config->MaxRows ;
 
     /* Pass 2: we have to decide on number of rows/columns : */
     WinListState.columns_num = WinListState.rows_num = 0 ;
@@ -705,7 +657,7 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
        * To resolve that race condition - lets try and find maximum number of columns that can fit into
        * allowed_max_width  - we start with min(windows_num,max_columns) columns and then decrease this number untill
        * we get a fit*/
-        int columns_num = min(WinListState.windows_num,Config->max_columns);
+        int columns_num = min(WinListState.windows_num,Config->MaxColumns);
         int max_row_width = 0;
         int min_columns = (WinListState.windows_num + max_rows - 1)/ max_rows ;
         if( min_columns <= 0 )
@@ -938,43 +890,159 @@ find_button_by_position( int x, int y )
 }
 
 /* Public stuff : ***************************************************/
+void 
+configure_tbar_icon( ASTBarData *tbar, ASWindowData *wd )
+{
+	ASRawHints    raw;
+	ASHints       clean;
+	ASSupportedHints *list = create_hints_list ();
+	ASImage *icon_im = NULL ;
+		
+	enable_hints_support (list, HINTS_ICCCM);
+	enable_hints_support (list, HINTS_KDE);
+	enable_hints_support (list, HINTS_ExtendedWM);
+	enable_hints_support (list, HINTS_ASDatabase);
+		
+	memset( &raw, 0x00, sizeof(ASRawHints));
+	memset( &clean, 0x00, sizeof(ASHints));
+		
+	if( collect_hints (ASDefaultScr, wd->client, HINT_NAME|HINT_GENERAL, &raw) )
+	{
+		if( merge_hints (&raw, Database, NULL, list, HINT_NAME|HINT_GENERAL, &clean, wd->client) )
+		{
+			icon_im = get_client_icon_image( ASDefaultScr, &clean );
+			destroy_hints( &clean, True );
+		}
+		destroy_raw_hints ( &raw, True);
+	}
+	destroy_hints_list( &list );		
+
+	if( icon_im ) 
+	{
+		int pos[10][2] = {{0,1},
+						  {0,2},{1,2},{2,2},
+						  {0,1},{1,1},{2,1},
+						  {0,0},{1,0},{2,0}};
+		int width = icon_im->width ; 
+	    int height = icon_im->height ; 
+
+		if( get_flags( Config->flags, WINLIST_ScaleIconToTextHeight ) )
+		{
+			 height = width = calculate_astbar_height( tbar );
+		}	 
+		if( get_flags(Config->set_flags, WINLIST_IconSize) )
+		{
+			if( Config->IconSize.width > 0 ) 
+				width = Config->IconSize.width;
+			if( Config->IconSize.height > 0 ) 
+				height = Config->IconSize.height;
+		}
+		if( width != icon_im->width || height != icon_im->height ) 
+		{
+			ASImage *scaled_im = scale_asimage( Scr.asv, icon_im, width, height, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
+			if( scaled_im != NULL ) 
+			{
+			   	safe_asimage_destroy( icon_im );
+				icon_im = scaled_im ;
+			}					   
+		}	 
+		add_astbar_icon( tbar, 
+						 pos[Config->IconLocation%10][0], 
+						 pos[Config->IconLocation%10][1], 
+						 0, Config->IconAlign, icon_im );
+   	   	safe_asimage_destroy( icon_im );
+	}	 
+}
+
+
 static void
 configure_tbar_props( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
 {
 	INT32 encoding = AS_Text_ASCII ;
-	char *name = get_window_name(wd, Config->show_name_type, &encoding );
-    ASFlagType align = ALIGN_TOP|ALIGN_BOTTOM ;
+	char *name = get_window_name(wd, Config->UseName, &encoding );
 
 	if( !focus_only ) 
 	{
+	    ASFlagType align = ALIGN_TOP|ALIGN_BOTTOM ;
+		int h_spacing = Config->HSpacing ;
+		int v_spacing = Config->VSpacing ;
+		int fbevel = Config->FBevel ;
+		int sbevel = Config->SBevel ;
+		int ubevel = Config->UBevel ;
+		int fake_label_id = -1 ;
+				   
     	delete_astbar_tile( tbar, -1 );
-    	tbar->h_border = Config->h_spacing ;
-    	tbar->v_border = Config->v_spacing ;
+		LOCAL_DEBUG_OUT( "setting bar border to %+d, %+d", tbar->h_border, tbar->v_border );
     	set_astbar_style_ptr( tbar, BAR_STATE_FOCUSED, Scr.Look.MSWindow[BACK_FOCUSED] );
-    	set_astbar_hilite( tbar, BACK_FOCUSED, Config->fbevel );
-    	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->fcm );
+		if( WinListState.tbar_props )
+		{
+			if( !get_flags( Config->set_flags, WINLIST_Align ) )
+				align = WinListState.tbar_props->align ;
+			else
+				align = Config->Align ;
+		    if( !get_flags( Config->set_flags, WINLIST_HSpacing) )
+				h_spacing = WinListState.tbar_props->title_h_spacing ;
+    		if( !get_flags( Config->set_flags, WINLIST_VSpacing) )
+    			v_spacing = WinListState.tbar_props->title_v_spacing ;
+			if( !get_flags( Config->set_flags, WINLIST_FBevel ) )
+			{	
+				fbevel = WinListState.tbar_props->bevel ;
+				if( fbevel == 0  ) 
+					fbevel = DEFAULT_TBAR_HILITE ;
+			}
+			if( !get_flags( Config->set_flags, WINLIST_SBevel ) )
+			{	
+				sbevel = WinListState.tbar_props->bevel ;
+				if( sbevel == 0  ) 
+					sbevel = DEFAULT_TBAR_HILITE ;
+			}
+
+			if( !get_flags( Config->set_flags, WINLIST_UBevel ) )
+			{	
+				ubevel = WinListState.tbar_props->bevel ;
+				if( fbevel == 0  ) 
+					fbevel = DEFAULT_TBAR_HILITE ;
+			}
+
+		}	 
+
+		if( sbevel == 0  ) 
+			sbevel = DEFAULT_TBAR_HILITE ;
+
+    	set_astbar_hilite( tbar, BACK_FOCUSED, fbevel );
+    	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->FCompositionMethod );
     	if( get_flags(wd->state_flags, AS_Sticky) )
     	{
         	set_astbar_style_ptr( tbar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_STICKY] );
-        	set_astbar_hilite( tbar, BACK_UNFOCUSED, Config->sbevel );
-        	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->scm );
+        	set_astbar_hilite( tbar, BACK_UNFOCUSED, sbevel );
+        	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->SCompositionMethod );
     	}else
     	{
         	set_astbar_style_ptr( tbar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_UNFOCUSED] );
-        	set_astbar_hilite( tbar, BACK_UNFOCUSED, Config->ubevel );
-        	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->ucm );
+        	set_astbar_hilite( tbar, BACK_UNFOCUSED, ubevel );
+        	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->UCompositionMethod );
     	}
-
-    	align = Config->name_aligment ;
+		
+		if( get_flags( Config->flags, ASWL_ShowIcon ) && (Config->IconLocation%10 == 5 ))
+		{	
+        	fake_label_id = add_astbar_label( tbar, 3, 3, 0, align, h_spacing, v_spacing, "Aq#,`", 0);
+			configure_tbar_icon( tbar, wd );
+		}
     	if( get_flags( wd->state_flags, AS_Iconic ) && name != NULL && name[0] != '\0')
     	{
         	char *iconic_name = safemalloc(1+strlen(name)+1+1);
         	sprintf(iconic_name, "(%s)", name );
-        	add_astbar_label( tbar, 0, 0, 0, align, 0, 0, iconic_name, encoding);
+        	add_astbar_label( tbar, 1, 1, 0, align, h_spacing, v_spacing, iconic_name, encoding);
         	free( iconic_name );
     	}else
-        	add_astbar_label( tbar, 0, 0, 0, align, 0, 0, name, encoding);
-    	set_astbar_balloon( tbar, 0, name, encoding );
+        	add_astbar_label( tbar, 1, 1, 0, align, h_spacing, v_spacing, name, encoding);
+    	
+		if( fake_label_id > -1 ) 
+			delete_astbar_tile( tbar, fake_label_id );
+		else if( get_flags( Config->flags, ASWL_ShowIcon ) )
+			configure_tbar_icon( tbar, wd );
+
+		set_astbar_balloon( tbar, 0, name, encoding );
 	}
     set_astbar_focused( tbar, WinListState.main_canvas, wd->focused );
     if( wd->focused )
@@ -1096,7 +1164,7 @@ release_winlist_button( ASWindowData *wd, int button )
             update_canvas_display( WinListState.main_canvas );
 		}
 
-        action_list = Config->mouse_actions[button - Button1] ;
+        action_list = Config->Action[button - Button1] ;
         if( action_list )
         {
             int i = -1;
