@@ -29,6 +29,7 @@
 #include <signal.h>
 #include <unistd.h>
 
+/* #include <X11/keysym.h> */
 #include "../../libAfterImage/afterimage.h"
 
 #include "../../libAfterStep/afterstep.h"
@@ -80,6 +81,9 @@ typedef struct {
 #define ASWT_AllDesks		(0x01<<2)
 #define ASWT_Transparent	(0x01<<3)
 #define ASWT_ShutDownInProgress	(0x01<<4)
+#define ASWT_SkipTransients		(0x01<<5)
+#define ASWT_StateShaded		(0x01<<6)
+#define ASWT_StateSticky		(0x01<<7)
 
 	ASFlagType flags ;
 
@@ -111,6 +115,8 @@ typedef struct {
 	MyButton unswallow_button ;
 
 	CARD32      my_desktop ;
+	
+	ASHashTable *unswallowed_apps ; 
 
 }ASWinTabsState ;
 
@@ -120,10 +126,12 @@ ASWinTabsState WinTabsState = { 0 };
 #define C_UnswallowButton 	C_TButton1
 #define C_MenuButton 		C_TButton2
 
-
+#define WINTABS_SWITCH_KEYCODE	  49	/* XKeysymToKeycode (dpy, XK_grave) */
+#define WINTABS_SWITCH_MOD        Mod1Mask
 #define WINTABS_TAB_EVENT_MASK    (ButtonReleaseMask | ButtonPressMask | \
 	                               LeaveWindowMask   | EnterWindowMask | \
-                                   StructureNotifyMask|PointerMotionMask )
+                                   StructureNotifyMask|PointerMotionMask| \
+								   	KeyPressMask )
 
 #define WINTABS_MESSAGE_MASK      (M_END_WINDOWLIST |M_DESTROY_WINDOW |M_SWALLOW_WINDOW| \
 					   			   WINDOW_CONFIG_MASK|WINDOW_NAME_MASK|M_SHUTDOWN)
@@ -139,6 +147,10 @@ ASWinTabsState WinTabsState = { 0 };
 WinTabsConfig *Config = NULL ;
 /**********************************************************************/
 
+char *pattern_override = NULL ;
+char *exclude_pattern_override = NULL ;
+char *title_override = NULL, *icon_title_override = NULL ;
+
 void CheckConfigSanity(const char *pattern_override, const char *exclude_pattern_override, 
 					   const char *title_override, const char *icon_title_override);
 void GetBaseOptions (const char *filename);
@@ -169,6 +181,10 @@ Bool unswallow_current_tab();
 Bool unswallow_tab(int t);
 void  update_focus();
 Bool handle_tab_name_change( Window client );
+void update_tabs_desktop();
+void update_tabs_state();
+void register_unswallowed_client( Window client );
+Bool check_unswallowed_client( Window client ); 
 
 /* above function may also return : */
 #define BANNER_TAB_INDEX -1		   
@@ -177,14 +193,30 @@ Bool handle_tab_name_change( Window client );
 
 void DeadPipe(int);
 
-CommandLineOpts WinTabs_cmdl_options[6] =
+CommandLineOpts WinTabs_cmdl_options[8] =
 {
-	{NULL, "pattern","Overrides module's inclusion pattern", NULL, handler_set_string, NULL, 0, CMO_HasArgs },
-	{NULL, "exclude-pattern","Overrides module's exclusion pattern", NULL, handler_set_string, NULL, 0, CMO_HasArgs },
-	{NULL, "all-desks","Swallow windows from any desktop", NULL, handler_set_flag, NULL, 0, 0 },
-	{NULL, "title","Overrides module's main window title", NULL, handler_set_string, NULL, 0, CMO_HasArgs },
-	{NULL, "icon-title","Overrides module's title in iconic state", NULL, handler_set_string, NULL, 0, CMO_HasArgs },
-    {NULL, NULL, NULL, NULL, NULL, NULL, 0 }
+	{NULL, "pattern","Overrides module's inclusion pattern", NULL,
+	 handler_set_string, &pattern_override, 0, CMO_HasArgs },
+	
+	{NULL, "exclude-pattern","Overrides module's exclusion pattern", NULL,
+	 handler_set_string, &exclude_pattern_override, 0, CMO_HasArgs },
+	
+	{NULL, "all-desks","Swallow windows from any desktop", NULL, handler_set_flag, &(WinTabsState.flags),
+	 ASWT_AllDesks, 0 },
+	
+	{NULL, "title","Overrides module's main window title", NULL, handler_set_string,
+	 &title_override, 0, CMO_HasArgs },
+	
+	{NULL, "icon-title","Overrides module's title in iconic state", NULL, handler_set_string,
+	 &icon_title_override, 0, CMO_HasArgs },
+	
+	{NULL, "skip-transients","Disregard any transient window(dialog)", NULL, handler_set_flag,
+	 &(WinTabsState.flags), ASWT_SkipTransients, 0 },
+    
+	{"tr", "transparent","keep window-background transparent", NULL, handler_set_flag,
+	 &(WinTabsState.flags), ASWT_Transparent, 0 },
+
+	{NULL, NULL, NULL, NULL, NULL, NULL, 0, 0 }
 };
 
 
@@ -220,9 +252,7 @@ int
 main( int argc, char **argv )
 {
     int i ;
-	char *pattern_override = NULL ;
-	char *exclude_pattern_override = NULL ;
-	char *title_override = NULL, *icon_title_override = NULL ;
+    register int opt;
 	/* Save our program name - for error messages */
 	set_DeadPipe_handler(DeadPipe);
     InitMyApp (CLASS_GADGET, argc, argv, WinTabs_usage, NULL, ASS_Restarting );
@@ -230,26 +260,28 @@ main( int argc, char **argv )
 
     set_signal_handler( SIGSEGV );
 	set_flags( WinTabsState.flags, ASWT_Transparent );  /* default */
-    for( i = 1 ; i< argc ; ++i)
-	{
-		LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
-		if( argv[i] != NULL )
-		{ 	
-	    	if( strcmp( argv[i] , "--pattern" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
-				pattern_override = argv[i+1];
-	    	else if( strcmp( argv[i] , "--exclude-pattern" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
-				exclude_pattern_override = argv[i+1];
-			else if( strcmp( argv[i] , "--all-desks" ) == 0 || strcmp( argv[i] , "-alldesks" ) == 0 )
-				set_flags( WinTabsState.flags, ASWT_AllDesks );
-			else if( strcmp( argv[i] , "--transparent" ) == 0 || strcmp( argv[i] , "-tr" ) == 0 )
-				set_flags( WinTabsState.flags, ASWT_Transparent );
-	    	else if( strcmp( argv[i] , "--title" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
-				title_override = argv[i+1];
-	    	else if( strcmp( argv[i] , "--icon-title" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
-				icon_title_override = argv[i+1];
-		}
-	}
+    
 
+	
+for( i = 1 ; i< argc ; ++i)
+ {
+	 LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
+	 if( argv[i] != NULL )
+	 { 	
+		 if( ( opt = match_command_line_opt( &(argv[i][0]), WinTabs_cmdl_options ) ) < 0)
+			 continue;
+		 
+		 /* command-line-option 'opt' has been matched */
+		 
+		 if( get_flags ( WinTabs_cmdl_options[opt].flags, CMO_HasArgs) )
+			 if( ++i >= argc)
+				 continue;
+		 
+		 WinTabs_cmdl_options[opt].handler( argv[i], WinTabs_cmdl_options[opt].trg,
+						    WinTabs_cmdl_options[opt].param);
+		 
+	 }
+ }
 
     ConnectX( ASDefaultScr, EnterWindowMask );
     ConnectAfterStep ( WINTABS_MESSAGE_MASK, 0 );               /* no AfterStep */
@@ -385,13 +417,13 @@ CheckConfigSanity(const char *pattern_override, const char *exclude_pattern_over
 	if( MyArgs.geometry.flags != 0 ) 
 		Config->geometry = MyArgs.geometry ;
 	if( pattern_override ) 
-		set_string_value(&(Config->pattern), mystrdup(pattern_override), NULL, 0);
+		set_string(&(Config->pattern), mystrdup(pattern_override));
 	if( exclude_pattern_override ) 
-		set_string_value(&(Config->exclude_pattern), mystrdup(exclude_pattern_override), NULL, 0);
+		set_string(&(Config->exclude_pattern), mystrdup(exclude_pattern_override) );
    	if( title_override ) 
-		set_string_value(&(Config->title), mystrdup(title_override), NULL, 0 );
+		set_string(&(Config->title), mystrdup(title_override) );
    	if( icon_title_override ) 
-		set_string_value(&(Config->icon_title), mystrdup(icon_title_override), NULL, 0 );
+		set_string(&(Config->icon_title), mystrdup(icon_title_override) );
 
 	if( Config->icon_title == NULL && Config->title != NULL ) 
 	{
@@ -412,8 +444,17 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
 		show_warning( "Empty Pattern requested for windows to be captured and tabbed - will wait for swallow command");
 	}
 
-	if( get_flags(Config->flags, WINTABS_AllDesks ) )
-		set_flags( WinTabsState.flags, ASWT_AllDesks );		
+	if( get_flags(Config->set_flags, WINTABS_AllDesks ) )
+	{	
+		if( get_flags(Config->flags, WINTABS_AllDesks ) )
+			set_flags( WinTabsState.flags, ASWT_AllDesks );		
+	}
+	if( get_flags(Config->set_flags, WINTABS_SkipTransients ) )
+	{	
+		if( get_flags(Config->flags, WINTABS_SkipTransients ) )
+			set_flags( WinTabsState.flags, ASWT_SkipTransients );		
+	}
+
 
     if( !get_flags(Config->geometry.flags, WidthValue) )
 		Config->geometry.width = 640 ;
@@ -460,20 +501,7 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
     Print_balloonConfig ( Config->balloon_conf );
 #endif
 	
-	balloon_config2look( &(Scr.Look), Config->balloon_conf );
-	if( Config->balloon_conf == NULL ) 
-	{
-/*		
-		Scr.Look.balloon_look->show = True ;
-        Scr.Look.balloon_look->xoffset = 5 ;
-        Scr.Look.balloon_look->yoffset = 5 ;
-		Scr.Look.balloon_look->style = mystyle_list_find_or_default (NULL, NULL);
-		Scr.Look.balloon_look->close_delay = 2000 ;
- */
-	}	 
-		
-    LOCAL_DEBUG_OUT( "balloon mystyle = %p (\"%s\")", Scr.Look.balloon_look->style,
-                    Scr.Look.balloon_look->style?Scr.Look.balloon_look->style->name:"none" );
+	balloon_config2look( &(Scr.Look), Config->balloon_conf, "*WinTabsBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
 
 }
@@ -506,7 +534,7 @@ GetOptions (const char *filename)
 
     if( config->pattern )
     {    
-        set_string_value( &(Config->pattern), mystrdup(config->pattern), NULL, 0 );
+        set_string( &(Config->pattern), mystrdup(config->pattern) );
         Config->pattern_type = config->pattern_type ; 
     }
     if( get_flags(config->set_flags, WINTABS_MaxRows) )
@@ -519,11 +547,11 @@ GetOptions (const char *filename)
         Config->max_tab_width = config->max_tab_width;
 
     if( config->unfocused_style )
-        set_string_value( &(Config->unfocused_style), mystrdup(config->unfocused_style), NULL, 0 );
+        set_string( &(Config->unfocused_style), mystrdup(config->unfocused_style) );
     if( config->focused_style )
-        set_string_value( &(Config->focused_style), mystrdup(config->focused_style), NULL, 0 );
+        set_string( &(Config->focused_style), mystrdup(config->focused_style) );
     if( config->sticky_style )
-        set_string_value( &(Config->sticky_style), mystrdup(config->sticky_style), NULL, 0 );
+        set_string( &(Config->sticky_style), mystrdup(config->sticky_style) );
 
     if( get_flags(config->set_flags, WINTABS_Align) )
         Config->name_aligment = config->name_aligment;
@@ -657,7 +685,13 @@ DispatchEvent (ASEvent * event)
 		int pointer_root_x = event->x.xkey.x_root;
 		int pointer_root_y = event->x.xkey.y_root;
 		static int last_pointer_root_x = -1, last_pointer_root_y = -1; 
-        
+	
+		if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+		{
+			withdraw_balloon(NULL);
+			return;
+		}
+
 		pointer_tab  = find_tab_by_position( pointer_root_x, pointer_root_y );
         LOCAL_DEBUG_OUT( "pointer at %d,%d - pointer_tab = %d", event->x.xmotion.x_root, event->x.xmotion.y_root, pointer_tab );
 		if( pointer_tab == BANNER_TAB_INDEX ) 
@@ -716,6 +750,29 @@ DispatchEvent (ASEvent * event)
                     render_tabs( get_flags( rerender_tabs, CANVAS_RESIZED ));
             }
 	        break;
+        case KeyPress:
+			{
+				XKeyEvent *xk = &(event->x.xkey);	
+				if( xk->keycode == WINTABS_SWITCH_KEYCODE && xk->state == WINTABS_SWITCH_MOD ) 
+				{	
+					if( WinTabsState.selected_tab+1 < PVECTOR_USED( WinTabsState.tabs ) )
+				 		select_tab( WinTabsState.selected_tab+1 );    		
+					else
+						select_tab( 0 );    		   
+					/* XBell (dpy, event->scr->screen); */
+				}else if( xk->window != WinTabsState.tabs_window ) 
+				{	
+					if( WinTabsState.selected_tab >= 0 && WinTabsState.selected_tab < PVECTOR_USED( WinTabsState.tabs ) )
+    				{
+        				ASWinTab *old_tab =  PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+WinTabsState.selected_tab ;
+	
+						xk->window = old_tab->client;
+        	    		XSendEvent (dpy, xk->window, False, KeyPressMask, &(event->x));
+					}
+				}
+
+			}	 
+			break;
         case ButtonPress:
             if( pointer_tab >= 0 ) 
                 press_tab( pointer_tab );
@@ -847,20 +904,45 @@ DispatchEvent (ASEvent * event)
 						XSelectInput( dpy, Scr.Root, EnterWindowMask );
 					}
 				}	 
+			}else if(   event->w == WinTabsState.main_window )
+			{
+			 	if( event->x.xproperty.atom == _XA_NET_WM_DESKTOP ) 
+				{
+					CARD32 new_desk = read_extwm_desktop_val(WinTabsState.main_window);
+
+					if( !get_flags( WinTabsState.flags, ASWT_AllDesks )) 
+					{
+						if( WinTabsState.my_desktop != new_desk )
+						{	
+							WinTabsState.my_desktop = new_desk ;
+							if( WinTabsState.pattern_wrexp != NULL ) 
+								iterate_window_data( recheck_swallow_windows, NULL);
+						}
+					}else
+						WinTabsState.my_desktop = new_desk ;
+
+					update_tabs_desktop();
+				}else if( event->x.xproperty.atom == _XA_NET_WM_STATE ) 
+				{
+					ASFlagType extwm_flags = 0 ; 
+					if( get_extwm_state_flags (WinTabsState.main_window, &extwm_flags) )
+					{
+						ASFlagType new_state = get_flags(extwm_flags, EXTWM_StateShaded)?ASWT_StateShaded:0;
+
+						new_state |= get_flags(extwm_flags, EXTWM_StateSticky)?ASWT_StateSticky:0;
+						LOCAL_DEBUG_OUT( "old_state = %lX, new_state = %lX", (WinTabsState.flags&(ASWT_StateShaded|ASWT_StateSticky)), new_state );
+						if( (WinTabsState.flags&(ASWT_StateShaded|ASWT_StateSticky)) != new_state )
+						{
+							clear_flags( WinTabsState.flags, ASWT_StateShaded|ASWT_StateSticky );
+							set_flags( WinTabsState.flags, new_state );
+							update_tabs_state();
+						}		  
+					}		
+					
+				}	 
 			}else if( IsNameProp(event->x.xproperty.atom) )
 			{                  /* Maybe name change on the client !!! */
 				handle_tab_name_change( event->w );				 		
-			}else if(   event->x.xproperty.atom == _XA_NET_WM_DESKTOP && 
-						event->w == WinTabsState.main_window && 
-						!get_flags( WinTabsState.flags, ASWT_AllDesks )) 
-			{
-				CARD32 new_desk = read_extwm_desktop_val(WinTabsState.main_window);
-				if( WinTabsState.my_desktop != new_desk )
-				{	
-					WinTabsState.my_desktop = new_desk ;
-					if( WinTabsState.pattern_wrexp != NULL ) 
-						iterate_window_data( recheck_swallow_windows, NULL);
-				}
 			}	 
 			break;
 		default:
@@ -992,10 +1074,11 @@ make_wintabs_window()
     extwm_hints.flags = EXTWM_PID|EXTWM_TypeASModule|EXTWM_TypeNormal ;
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
+	set_client_cmd (w);
 
     /* we will need to wait for PropertyNotify event indicating transition
 	   into Withdrawn state, so selecting event mask: */
-    XSelectInput (dpy, w, PropertyChangeMask|StructureNotifyMask|FocusChangeMask 
+    XSelectInput (dpy, w, PropertyChangeMask|StructureNotifyMask|FocusChangeMask|KeyPressMask 
                           /*|ButtonReleaseMask | ButtonPressMask */
                   );
 
@@ -1017,6 +1100,7 @@ make_tabs_window( Window parent )
     Window w ;
 	attr.event_mask = WINTABS_TAB_EVENT_MASK ;
     w = create_visual_window( Scr.asv, parent, 0, 0, 1, 1, 0, InputOutput, CWEventMask, &attr );
+	XGrabKey( dpy, WINTABS_SWITCH_KEYCODE, WINTABS_SWITCH_MOD, w, True, GrabModeAsync, GrabModeAsync);
     return w;
 }
 
@@ -1026,7 +1110,7 @@ make_frame_window( Window parent )
 	static XSetWindowAttributes attr ;
 	ASFlagType attr_mask ;
     Window w ;
-	attr.event_mask = SubstructureRedirectMask|FocusChangeMask ;
+	attr.event_mask = SubstructureRedirectMask|FocusChangeMask|KeyPressMask ;
 	attr_mask = CWEventMask ;
 	if( get_flags( WinTabsState.flags, ASWT_Transparent ) )
 	{
@@ -1040,6 +1124,7 @@ make_frame_window( Window parent )
 		attr_mask |= CWBackPixel ;
 	}
     w = create_visual_window( Scr.asv, parent, 0, 0, WinTabsState.win_width, WinTabsState.win_height, 0, InputOutput, attr_mask, &attr );
+	XGrabKey( dpy, WINTABS_SWITCH_KEYCODE, WINTABS_SWITCH_MOD, w, True, GrabModeAsync, GrabModeAsync);
     return w;
 }
 
@@ -1160,7 +1245,10 @@ void
 place_tabs_line( ASWinTab *tabs, int x, int y, int first, int last, int spare, int max_width, int tab_height )
 {
     int i ;
-    int delta = spare / (last+1-first) ;
+    int delta = 0;
+	
+	if( last+1 > first  ) 
+		delta = spare / (last+1-first) ;
 
     for( i = first ; i <= last ; ++i ) 
     {
@@ -1197,13 +1285,19 @@ moveresize_client( ASWinTab *aswt, int x, int y, int width, int height )
 	if( get_flags( aswt->hints.flags, AS_SizeInc ) )
 	{
 		int min_w = 0, min_h = 0 ; 
+		if( aswt->hints.width_inc == 0 ) 
+			aswt->hints.width_inc = 1;
+		if( aswt->hints.height_inc == 0 ) 
+			aswt->hints.height_inc = 1;
 		if( get_flags( aswt->hints.flags, AS_MinSize ) )
 		{
 			min_w = aswt->hints.min_width ;
 			min_h = aswt->hints.min_height ;
 		}	 
 		if( width > min_w && aswt->hints.width_inc < width  ) 
+		{
 			width = min_w + ((width - min_w)/aswt->hints.width_inc)*aswt->hints.width_inc ;
+		}
 
 		if( height > min_h && aswt->hints.height_inc < height  ) 
 			height = min_h + ((height - min_h)/aswt->hints.height_inc)*aswt->hints.height_inc ;
@@ -1328,8 +1422,8 @@ rearrange_tabs( Bool dont_resize_window )
     if( i >= tabs_num )    
         y += tab_height ; 
     
-    if( (moveresize_canvas( WinTabsState.tabs_canvas, 0, 0, max_x, y )&CANVAS_RESIZED)!= 0 );
-		render_tabs(True);
+    moveresize_canvas( WinTabsState.tabs_canvas, 0, 0, max_x, y );
+	render_tabs(True);
     
     max_y -= y ;
 	if( max_y <= 0 ) 
@@ -1375,12 +1469,18 @@ void
 select_tab( int tab )
 {
 	ASWinTab *new_tab ;
+	ASStatusHints status = {0};
+	ASFlagType default_status_flags = get_flags(WinTabsState.flags,ASWT_StateShaded)?AS_Shaded:0;
+
+	default_status_flags |= get_flags(WinTabsState.flags,ASWT_StateSticky)?AS_Sticky:0;
 
     if( tab == WinTabsState.selected_tab ) 
         return ; 
     if( WinTabsState.selected_tab >= 0 && WinTabsState.selected_tab < PVECTOR_USED( WinTabsState.tabs ) )
     {
         ASWinTab *old_tab =  PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+WinTabsState.selected_tab ;
+		status.flags = AS_Hidden|default_status_flags ; 
+		set_client_state (old_tab->client, &status);
         set_astbar_focused(old_tab->bar, WinTabsState.tabs_canvas, False);
         WinTabsState.selected_tab = -1 ;
     }    
@@ -1388,6 +1488,8 @@ select_tab( int tab )
 		tab = 0 ;
 		
     new_tab = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+tab ;
+	status.flags = default_status_flags ; 
+	set_client_state (new_tab->client, &status);
     set_astbar_focused(new_tab->bar, WinTabsState.tabs_canvas, True);
     XRaiseWindow( dpy, new_tab->frame_canvas->w );
     WinTabsState.selected_tab = tab ;
@@ -1483,7 +1585,7 @@ do_swallow_window( ASWindowData *wd )
 
     /* first thing - we reparent window and its icon if there is any */
     nc = aswt->client_canvas = create_ascanvas_container( wd->client );
-	aswt->frame_canvas = create_ascanvas( make_frame_window(WinTabsState.main_window) );
+	aswt->frame_canvas = create_ascanvas_container( make_frame_window(WinTabsState.main_window) );
 	aswt->swallow_location.x = nc->root_x ; 
 	aswt->swallow_location.y = nc->root_y ; 
 	aswt->swallow_location.width = nc->width ; 
@@ -1494,6 +1596,10 @@ do_swallow_window( ASWindowData *wd )
     XReparentWindow( dpy, wd->client, aswt->frame_canvas->w, WinTabsState.win_width - nc->width, WinTabsState.win_height - nc->height );
     XSelectInput (dpy, wd->client, StructureNotifyMask|PropertyChangeMask|FocusChangeMask);
     XAddToSaveSet (dpy, wd->client);
+	register_unswallowed_client( wd->client );    
+
+	XGrabKey( dpy, WINTABS_SWITCH_KEYCODE, WINTABS_SWITCH_MOD, wd->client, True, GrabModeAsync, GrabModeAsync);	  
+	set_client_desktop (wd->client, WinTabsState.my_desktop );	 
 
 #if 0   /* TODO : implement support for icons : */
     if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) &&
@@ -1556,11 +1662,16 @@ check_swallow_window( ASWindowData *wd )
 	ASWinTab *aswt = NULL ;
 	int i = 0;
 
-    if( wd == NULL && !get_flags( wd->state_flags, AS_Mapped))
+    if( wd == NULL || !get_flags( wd->state_flags, AS_Mapped))
         return;
 
 	if( wd->client == WinTabsState.main_window )
 		return;
+	if( get_flags( WinTabsState.flags, ASWT_SkipTransients ) )
+	{
+		if( get_flags( wd->flags, AS_Transient ) )
+			return;
+	}	 
 	/* first lets check if we have already swallowed this one : */
 	i = PVECTOR_USED(WinTabsState.tabs);
 	aswt = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
@@ -1576,6 +1687,8 @@ check_swallow_window( ASWindowData *wd )
 	LOCAL_DEBUG_OUT( "name(\"%s\")->icon_name(\"%s\")->res_class(\"%s\")->res_name(\"%s\")",
                      wd->window_name, wd->icon_name, wd->res_class, wd->res_name );
 	
+	if( check_unswallowed_client( wd->client ) )
+		return;
 	if( get_flags( Config->set_flags, WINTABS_PatternType ) )
 	{	
 		if( !check_swallow_name(get_window_name( wd, Config->pattern_type, &encoding )) )
@@ -1684,7 +1797,7 @@ void close_current_tab()
 	if( tabs_num > 0 && curr >= 0 && curr < tabs_num ) 
 	{
 		if( tabs[curr].closed ) 
-			XDestroyWindow( dpy, tabs[curr].client); 	
+			XKillClient( dpy, tabs[curr].client); 	
 		else
 		{	
 			send_wm_protocol_request(tabs[curr].client, _XA_WM_DELETE_WINDOW, CurrentTime);
@@ -1700,6 +1813,11 @@ Bool unswallow_tab(int t)
 	LOCAL_DEBUG_OUT( "tab = %d, tabs_num = %d", t, tabs_num );
 	if( tabs_num > 0 && t >= 0 && t < tabs_num ) 
 	{
+		/* XGrabKey( dpy, WINTABS_SWITCH_KEYCODE, Mod1Mask|Mod2Mask, w, True, GrabModeAsync, GrabModeAsync); */
+		XDeleteProperty( dpy, tabs[t].client, _XA_NET_WM_STATE );
+		XDeleteProperty( dpy, tabs[t].client, _XA_WIN_STATE );
+		XDeleteProperty( dpy, tabs[t].client, _XA_NET_WM_DESKTOP );
+		XDeleteProperty( dpy, tabs[t].client, _XA_WIN_WORKSPACE );
 		XResizeWindow( dpy, tabs[t].client, tabs[t].swallow_location.width, tabs[t].swallow_location.height );
 		XReparentWindow( dpy, tabs[t].client, Scr.Root, tabs[t].swallow_location.x, tabs[t].swallow_location.y );
 		delete_tab( t ); 		
@@ -1751,6 +1869,7 @@ Bool handle_tab_name_change( Window client)
 		ASSupportedHints *list = create_hints_list ();
 		
 		enable_hints_support (list, HINTS_ICCCM);
+		enable_hints_support (list, HINTS_KDE);
 		enable_hints_support (list, HINTS_ExtendedWM);
 		
 		memset( &raw, 0x00, sizeof(ASRawHints));
@@ -1782,3 +1901,51 @@ Bool handle_tab_name_change( Window client)
 	return changed ;
 }	 
 
+void 
+update_tabs_desktop()
+{	
+	int i = PVECTOR_USED(WinTabsState.tabs);
+    ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
+	while( --i >= 0) 
+		set_client_desktop (tabs[i].client, WinTabsState.my_desktop );	
+}
+
+void 
+update_tabs_state()
+{	
+	ASStatusHints status = {0};
+	int i = PVECTOR_USED(WinTabsState.tabs);
+    ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
+	
+	status.flags = get_flags(WinTabsState.flags,ASWT_StateShaded)?AS_Shaded:0;
+	status.flags |= get_flags(WinTabsState.flags,ASWT_StateSticky)?AS_Sticky:0;
+
+	while( --i >= 0) 
+	{	
+		if( i == WinTabsState.selected_tab ) 
+			clear_flags( status.flags, AS_Hidden );
+		else
+			set_flags( status.flags, AS_Hidden );
+		set_client_state (tabs[i].client, &status);
+	}
+}
+
+void 
+register_unswallowed_client( Window client ) 
+{
+	if( WinTabsState.unswallowed_apps == NULL ) 
+		WinTabsState.unswallowed_apps = create_ashash( 0, NULL, NULL, NULL ); 	 
+
+	add_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), NULL );
+}
+
+
+Bool 
+check_unswallowed_client( Window client ) 
+{
+	ASHashData hdata = {0} ;
+	if( WinTabsState.unswallowed_apps != NULL ) 
+		return (get_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), &hdata.vptr ) == ASH_Success );
+	return False;
+}
+
