@@ -49,6 +49,8 @@
 #include "../../libAfterStep/session.h"
 #include "../../libAfterStep/mystyle_property.h"
 #include "../../libAfterStep/wmprops.h"
+#include "../../libAfterStep/desktop_category.h"
+#include "../../libAfterStep/kde.h"
 
 #include "../../libAfterConf/afterconf.h"
 
@@ -86,7 +88,6 @@ static int           MenuPinOnButton = -1 ;
 
 static MyStyleDefinition *MyStyleList = NULL ;
 static MyFrameDefinition *MyFrameList = NULL ;
-static char              *DefaultFrameName = NULL ;
 static MyFrameDefinition *LegacyFrameDef = NULL ;
 
 static balloonConfig BalloonConfig = {0, 0, 0, 0, 0, 0, NULL };
@@ -190,6 +191,7 @@ struct config main_config[] = {
     {"RecentSubmenuItems", SetInts, (char**)&TmpFeel.recent_submenu_items, (int*)&dummy},
 	{"WinListSortOrder", SetInts, (char**)&TmpFeel.winlist_sort_order, (int *)&dummy},
 	{"WinListHideIcons", SetFlag, (char**)WinListHideIcons, (int *)&dummy},
+	{"SuppressIcons", SetFlag, (char**)SuppressIcons, (int *)&dummy},
 
     /* look options */
 	/* obsolete stuff */
@@ -254,7 +256,7 @@ struct config main_config[] = {
     {"DeskBack"							, deskback_parse, NULL, NULL },
     {"*asetrootDeskBack"				, deskback_parse, NULL, NULL },        /* pretending to be asteroot here */
     {"MyFrame"							, myframe_parse, (char**)"afterstep", (int*)&MyFrameList},
-    {"DefaultFrame"						, assign_string, (char**)&DefaultFrameName, (int*)0},
+    {"DefaultFrame"						, assign_string, (char**)&TmpLook.DefaultFrameName, (int*)0},
     {"DontDrawBackground"				, SetLookFlag, (char **)DontDrawBackground, NULL},
     {"CursorFore"						, assign_string, &TmpLook.CursorFore, (int *)0},    /* foreground color to be used for coloring pointer's cursor */
     {"CursorBack"						, assign_string, &TmpLook.CursorBack, (int *)0},    /* background color to be used for coloring pointer's cursor */
@@ -262,6 +264,7 @@ struct config main_config[] = {
 	{"CustomCursor"						, SetCustomCursor, (char **)0, (int *)0},
 	{"Cursor"							, SetCursor, (char **)0, (int *)0},
     /***********************************/
+    {"IconsGrowVertically", SetLookFlag, (char **)IconsGrowVertically, (int *) 0},
 	{"MenuPinOn"						, assign_string, &MenuPinOn, (int *)0},    /* menu pin */
     {"MArrowPixmap"						, assign_pixmap, (char **)&TmpLook.MenuArrow, (int *)0},   /* menu arrow */
     {"TitlebarNoPush"					, SetLookFlag, (char **)TitlebarNoPush, NULL},
@@ -283,6 +286,7 @@ struct config main_config[] = {
     {"StartMenuSortMode"				, SetInts, (char **)&TmpLook.StartMenuSortMode, (int *)&dummy},
     {"DrawMenuBorders"					, SetInts, (char **)&TmpLook.DrawMenuBorders, (int *)&dummy},
     {"ButtonSize"						, SetInts, (char **)&TmpLook.ButtonWidth, (int *)&TmpLook.ButtonHeight},
+    {"MinipixmapSize"					, SetInts, (char **)&TmpLook.minipixmap_width, (int *)&TmpLook.minipixmap_height},
     {"SeparateButtonTitle"				, SetLookFlag, (char **)SeparateButtonTitle, NULL},
     {"RubberBand"						, SetInts, (char **)&TmpLook.RubberBand, &dummy},
     {"DefaultStyle"						, assign_string, (char **)&MSWindowName[BACK_DEFAULT], (int *)0},
@@ -299,12 +303,14 @@ struct config main_config[] = {
     {"MenuHiliteCompositionMethod"		, SetInts, (char **)&TmpLook.menu_hcm, &dummy},
     {"MenuStippleCompositionMethod"		, SetInts, (char **)&TmpLook.menu_scm, &dummy},
     {"ShadeAnimationSteps"				, SetInts, (char **)&TmpFeel.ShadeAnimationSteps, (int *)&dummy},
-    {"TitleButtonBalloonBorderHilite"	, bevel_parse, (char**)"afterstep", (int*)&(BalloonConfig.border_hilite)},
-    {"TitleButtonBalloonXOffset"		, SetInts, (char**)&(BalloonConfig.x_offset), NULL},
-    {"TitleButtonBalloonYOffset"		, SetInts, (char**)&(BalloonConfig.y_offset), NULL},
-    {"TitleButtonBalloonDelay"			, SetInts, (char**)&(BalloonConfig.delay), NULL},
-    {"TitleButtonBalloonCloseDelay"		, SetInts, (char**)&(BalloonConfig.close_delay), NULL},
-    {"TitleButtonBalloonStyle"			, assign_string, &(BalloonConfig.style), NULL},
+    {"TitleButtonBalloonBorderHilite"	, bevel_parse, (char**)"afterstep", (int*)&(BalloonConfig.BorderHilite)},
+    {"TitleButtonBalloonXOffset"		, SetInts, (char**)&(BalloonConfig.XOffset), NULL},
+    {"TitleButtonBalloonYOffset"		, SetInts, (char**)&(BalloonConfig.YOffset), NULL},
+    {"TitleButtonBalloonDelay"			, SetInts, (char**)&(BalloonConfig.Delay), NULL},
+    {"TitleButtonBalloonCloseDelay"		, SetInts, (char**)&(BalloonConfig.CloseDelay), NULL},
+    {"TitleButtonBalloonStyle"			, assign_string, &(BalloonConfig.Style), NULL},
+    {"TitleButtonBalloonTextPaddingX"	, SetInts, (char**)&(BalloonConfig.TextPaddingX), NULL},
+    {"TitleButtonBalloonTextPaddingY"	, SetInts, (char**)&(BalloonConfig.TextPaddingY), NULL},
     {"TitleButtonBalloons"				, SetFlag2, (char**)BALLOON_USED, (int*)&(BalloonConfig.set_flags)},
     {"TitleButton"						, SetTitleButton, (char **)1, (int *)0},
     {"KillBackgroundThreshold"			, SetInts, (char**)&(TmpLook.KillBackgroundThreshold), NULL },
@@ -312,6 +318,9 @@ struct config main_config[] = {
 	{"CoverAnimationSteps"				, SetInts, (char**)&(TmpFeel.desk_cover_animation_steps), NULL },
 	{"CoverAnimationType"				, SetInts, (char**)&(TmpFeel.desk_cover_animation_type), NULL },
 	{"AnimateDeskChange"				, SetFlag, (char**)AnimateDeskChange, &dummy },
+	{"ButtonIconSpacing" 				, SetInts, (char **)&TmpLook.ButtonIconSpacing, &dummy},
+    {"ButtonBevel"						, bevel_parse, (char**)"afterstep", (int*)&(TmpLook.ButtonBevel)},
+    {"ButtonAlign"						, align_parse, (char**)"afterstep", (int*)&(TmpLook.ButtonAlign)},
 	{"", 0, (char **)0, (int *)0}
 };
 
@@ -498,7 +507,11 @@ init_old_look_variables (Bool free_resources)
 void
 merge_old_look_variables (MyLook *look)
 {
-    char         *button_style_names[BACK_STYLES] = {"ButtonTitleFocus", "ButtonTitleUnfocus", "ButtonTitleSticky", "ButtonTitleDefault" };
+    char         *button_style_names[BACK_STYLES] = 
+{AS_ICON_TITLE_MYSTYLE, 
+													 AS_ICON_TITLE_UNFOCUS_MYSTYLE, 
+													 AS_ICON_TITLE_STICKY_MYSTYLE, 
+													 "ButtonTitleDefault" };
     MyStyle      *button_styles[BACK_STYLES];
     int i ;
 
@@ -605,11 +618,13 @@ merge_old_look_variables (MyLook *look)
 void
 InitFeel (ASFeel *feel, Bool free_resources)
 {
-    if (free_resources && feel)
-		destroy_asfeel( feel, True );
-    feel->magic = MAGIC_ASFEEL ;
-	init_asfeel( feel );
-
+	if( feel ) 
+	{	
+    	if (free_resources)
+			destroy_asfeel( feel, True );
+    	feel->magic = MAGIC_ASFEEL ;
+		init_asfeel( feel );
+	}
 }
 
 void 
@@ -652,6 +667,7 @@ InitLook (MyLook *look, Bool free_resources)
 {
 	int i ;
     /* actuall MyLook cleanup : */
+
     mylook_init (look, free_resources, LL_Everything );
 
     /* other related things : */
@@ -671,12 +687,10 @@ InitLook (MyLook *look, Bool free_resources)
         unload_font (&IconFont);
         DestroyMyStyleDefinitions (&MyStyleList);
         DestroyMyFrameDefinitions (&MyFrameList);
-        if( DefaultFrameName )
-            free( DefaultFrameName );
 		if( LegacyFrameDef )
 			DestroyMyFrameDefinitions( &LegacyFrameDef );
-        if( BalloonConfig.style )
-            free( BalloonConfig.style );
+        if( BalloonConfig.Style )
+            free( BalloonConfig.Style );
 		for( i = 0  ; i < BACK_STYLES ; ++i )
 			if( MSWindowName[i] )
 				free( MSWindowName[i] );
@@ -702,7 +716,6 @@ InitLook (MyLook *look, Bool free_resources)
 		MSMenuName[i] = NULL ;
 
 	MyFrameList = NULL ;
-    DefaultFrameName = NULL ;
 	LegacyFrameDef = NULL ;
     memset( &BalloonConfig, 0x00, sizeof(BalloonConfig));
 }
@@ -725,13 +738,30 @@ merge_look( MyLook *to, MyLook *from )
     to->resize_move_geometry = from->resize_move_geometry ; 
     to->StartMenuSortMode = from->StartMenuSortMode ; 
     to->DrawMenuBorders = from->DrawMenuBorders ; 
-    to->ButtonWidth  = from->ButtonWidth ; 
-	to->ButtonHeight = from->ButtonHeight ; 
+    to->ButtonWidth  = (from->ButtonWidth == 0)? 64 :from->ButtonWidth  ; 
+	to->ButtonHeight = (from->ButtonHeight== 0)? to->ButtonWidth :from->ButtonHeight ; 
+	to->ButtonIconSpacing = from->ButtonIconSpacing;
+	to->ButtonAlign  = from->ButtonAlign;
+	to->ButtonBevel  = from->ButtonBevel;
+
+    to->minipixmap_width  = (from->minipixmap_width == 0)? 24 : from->minipixmap_width ; 
+	to->minipixmap_height = (from->minipixmap_height == 0)?to->minipixmap_width:from->minipixmap_height; 
+	
+	asxml_var_insert("icon.button.width", to->ButtonWidth);
+	asxml_var_insert("icon.button.height", to->ButtonHeight);
+
+	asxml_var_insert("minipixmap.width", to->minipixmap_width);
+	asxml_var_insert("minipixmap.height", to->minipixmap_height);
+
+	to->DefaultFrameName = from->DefaultFrameName ; 
+
     to->RubberBand = from->RubberBand ; 
     to->menu_icm = from->menu_icm ; 
     to->menu_hcm = from->menu_hcm ; 
     to->menu_scm = from->menu_scm ;
     to->KillBackgroundThreshold = from->KillBackgroundThreshold ; 
+	to->desktop_animation_tint = from->desktop_animation_tint ;
+	LOCAL_DEBUG_OUT( "desk_anime_tint = %lX", from->desktop_animation_tint );
 }
 
 
@@ -778,9 +808,13 @@ make_styles (MyLook *look)
 
 MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagType default_title_align )
 {
-    MyFrame *frame = create_myframe();
+    MyFrame *frame;
     int i ;
-    frame->name = mystrdup( fd->name );
+
+	frame = get_flags(fd->flags,MYFRAME_INHERIT_DEFAULTS)? 
+				create_default_myframe(default_title_align) : create_myframe();
+    
+	frame->name = mystrdup( fd->name );
     for( i = 0 ; i < fd->inheritance_num ; ++i )
     {
 		ASHashData hdata ;
@@ -793,17 +827,17 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
     for( i = 0 ; i < FRAME_PARTS ; ++i )
     {
         if( fd->parts[i] )
-            set_string_value(&(frame->part_filenames[i]), mystrdup(fd->parts[i]), NULL, 0);
+            set_string(&(frame->part_filenames[i]), mystrdup(fd->parts[i]));
         if( get_flags( fd->set_part_size, 0x01<<i ) )
         {
-            frame->part_width[i] = fd->part_width[i];
-            frame->part_length[i] = fd->part_length[i];
+            frame->part_width[i] = max(fd->part_width[i],1);
+            frame->part_length[i] = max(fd->part_length[i],1);
         }
-        if( IsPartFBevelSet(frame,i) )
+        if( IsPartFBevelSet(fd,i) )
             frame->part_fbevel[i] = fd->part_fbevel[i];
-        if( IsPartUBevelSet(frame,i) )
+        if( IsPartUBevelSet(fd,i) )
             frame->part_ubevel[i] = fd->part_ubevel[i];
-        if( IsPartSBevelSet(frame,i) )
+        if( IsPartSBevelSet(fd,i) )
             frame->part_sbevel[i] = fd->part_sbevel[i];
         if( get_flags( fd->set_part_align, 0x01<<i ) )
             frame->part_align[i] = fd->part_align[i];
@@ -823,9 +857,9 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
         if( fd->title_styles[i] )
-            set_string_value(&(frame->title_style_names[i]), mystrdup(fd->title_styles[i]), NULL, 0 );
+            set_string(&(frame->title_style_names[i]), mystrdup(fd->title_styles[i]) );
         if( fd->frame_styles[i] )
-            set_string_value(&(frame->frame_style_names[i]), mystrdup(fd->frame_styles[i]), NULL, 0 );
+            set_string(&(frame->frame_style_names[i]), mystrdup(fd->frame_styles[i]) );
     }
     if( get_flags( fd->set_title_attr, MYFRAME_TitleFBevelSet ) )
         frame->title_fbevel = fd->title_fbevel;
@@ -835,6 +869,11 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
         frame->title_sbevel = fd->title_sbevel;
     if( get_flags( fd->set_title_attr, MYFRAME_TitleAlignSet ) )
         frame->title_align = fd->title_align;
+	if( get_flags( fd->set_title_attr, MYFRAME_LeftBtnAlignSet ) )
+        frame->left_btn_align = fd->left_btn_align;
+	if( get_flags( fd->set_title_attr, MYFRAME_RightBtnAlignSet ) )
+        frame->right_btn_align = fd->right_btn_align;
+
     if( get_flags( fd->set_title_attr, MYFRAME_CondenseTitlebarSet ) )
         frame->condense_titlebar = fd->condense_titlebar;
     if( get_flags( fd->set_title_attr, MYFRAME_LeftTitlebarLayoutSet ) )
@@ -844,12 +883,33 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
 	}
     if( get_flags( fd->set_title_attr, MYFRAME_RightTitlebarLayoutSet ) )
         frame->right_layout = fd->right_layout;
-    if( get_flags( fd->set_title_attr, MYFRAME_TitleFCMSet ) )
+	if( get_flags( fd->set_title_attr, MYFRAME_TitleFCMSet ) )
         frame->title_fcm = fd->title_fcm;
     if( get_flags( fd->set_title_attr, MYFRAME_TitleUCMSet ) )
         frame->title_ucm = fd->title_ucm;
     if( get_flags( fd->set_title_attr, MYFRAME_TitleSCMSet ) )
         frame->title_scm = fd->title_scm;
+
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleFHueSet ) )
+		parse_hue( fd->title_fhue, &(frame->title_fhue) );
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleUHueSet ) )
+        parse_hue( fd->title_uhue, &(frame->title_uhue) );
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleSHueSet ) )
+        parse_hue( fd->title_shue, &(frame->title_shue) );
+	
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleFSatSet ) )
+        frame->title_fsat = fd->title_fsat;
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleUSatSet ) )
+        frame->title_usat = fd->title_usat;
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleSSatSet ) )
+        frame->title_ssat = fd->title_ssat;
+	
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleHSpacingSet ) )
+		frame->title_h_spacing = fd->title_h_spacing;
+	if( get_flags( fd->set_title_attr, MYFRAME_TitleVSpacingSet ) )
+		frame->title_v_spacing = fd->title_v_spacing;
+
+	
 	for( i = 0 ; i < MYFRAME_TITLE_BACKS ; ++i )
 	{
 		if( get_flags( fd->set_title_attr, MYFRAME_TitleBackAlignSet_Start<<i ) )
@@ -857,7 +917,7 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
 
     	if( fd->title_backs[i] )
     	{
-        	set_string_value(&(frame->title_back_filenames[i]), mystrdup(fd->title_backs[i]), NULL, 0 );
+        	set_string(&(frame->title_back_filenames[i]), mystrdup(fd->title_backs[i]) );
         	if( !get_flags( fd->set_title_attr, MYFRAME_TitleBackAlignSet_Start<<i ) )
         	{
             	frame->title_backs_align[i] = FIT_LABEL_WIDTH ;
@@ -877,6 +937,12 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
         frame->title_sbevel = DEFAULT_TBAR_HILITE;
     if( !get_flags(frame->set_title_attr, MYFRAME_TitleAlignSet ) )
         frame->title_align = default_title_align;
+	
+	if( !get_flags( frame->set_title_attr, MYFRAME_LeftBtnAlignSet ) )
+        frame->left_btn_align = ALIGN_VCENTER;
+	if( !get_flags( frame->set_title_attr, MYFRAME_RightBtnAlignSet ) )
+        frame->right_btn_align = ALIGN_VCENTER;
+
     set_flags( frame->set_title_attr, MYFRAME_TitleBevelSet|MYFRAME_TitleAlignSet );
 
 	frame->set_flags = fd->set_flags ;
@@ -960,6 +1026,8 @@ void
 FixLook( MyLook *look )
 {
     ASFlagType default_title_align = ALIGN_LEFT ;
+	int tbar_font_size = 0 ;
+	int menu_font_size = 0 ;
     int i ;
 #ifdef LOCAL_DEBUG
     LOCAL_DEBUG_OUT( "syncing %s","");
@@ -1000,6 +1068,21 @@ FixLook( MyLook *look )
     ASSync(False);
 #endif
 
+	for( i = 0 ; i < BACK_STYLES ; ++i ) 
+		if( look->MSWindow[i] )
+			if( look->MSWindow[i]->font.as_font )
+				if( look->MSWindow[i]->font.as_font->max_height > tbar_font_size ) 
+					tbar_font_size = look->MSWindow[i]->font.as_font->max_height ;
+	asxml_var_insert("title.font.size", tbar_font_size);
+	
+	for( i = 0 ; i < MENU_BACK_STYLES ; ++i ) 
+		if( look->MSMenu[i] )
+			if( look->MSMenu[i]->font.as_font )
+				if( look->MSMenu[i]->font.as_font->max_height > menu_font_size ) 
+					menu_font_size = look->MSMenu[i]->font.as_font->max_height ;
+	asxml_var_insert("menu.font.size", menu_font_size);
+
+
     mystyle_list_set_property (Scr.wmprops, look->styles_list);
 #ifdef LOCAL_DEBUG
     LOCAL_DEBUG_OUT( "syncing %s","");
@@ -1011,7 +1094,11 @@ FixLook( MyLook *look )
     else if(look->TitleTextAlign == JUSTIFY_CENTER )
         default_title_align = ALIGN_CENTER ;
 
-    /* update frame geometries */
+	if( look->DefaultFrameName == NULL ) 
+		look->DefaultFrameName = mystrdup("default");
+    check_myframes_list( look );
+    
+	/* update frame geometries */
     if (get_flags( look->flags, DecorateFrames))
     {
         MyFrameDefinition *fd ;
@@ -1021,35 +1108,31 @@ FixLook( MyLook *look )
         PrintMyFrameDefinitions (MyFrameList, 1);
 #endif
         LOCAL_DEBUG_OUT( "MyFrameList %p", MyFrameList );
-        if( MyFrameList || LegacyFrameDef )
-            check_myframes_list( look );
         for( fd = MyFrameList ; fd != NULL ; fd = fd->next )
         {
             LOCAL_DEBUG_OUT( "processing MyFrameDefinition %p", fd );
-            frame = add_myframe_from_def( look->FramesList, fd, default_title_align|ALIGN_VCENTER );
-            myframe_load ( frame, Scr.image_manager );
+            if( (frame = add_myframe_from_def( look->FramesList, fd, default_title_align|ALIGN_VCENTER )) != NULL ) 
+            	myframe_load ( frame, Scr.image_manager );
         }
-		if( LegacyFrameDef && DefaultFrameName  == NULL )
+		if( LegacyFrameDef )
 		{
             LOCAL_DEBUG_OUT( "processing legacy MyFrameDefinition %p", LegacyFrameDef );
-            frame = add_myframe_from_def( look->FramesList, LegacyFrameDef, default_title_align|ALIGN_VCENTER );
-            myframe_load ( frame, Scr.image_manager );
-			if( look->DefaultFrame == NULL )
-				look->DefaultFrame = frame ;
+			LegacyFrameDef->name = mystrdup(look->DefaultFrameName);
+            if( (frame = add_myframe_from_def( look->FramesList, LegacyFrameDef, default_title_align|ALIGN_VCENTER )) != NULL )
+            	myframe_load ( frame, Scr.image_manager );
 		}
         DestroyMyFrameDefinitions (&MyFrameList);
 		DestroyMyFrameDefinitions (&LegacyFrameDef);
-        if( look->DefaultFrame == NULL && DefaultFrameName != NULL )
-        {
-			ASHashData hdata ;
-            if( get_hash_item( look->FramesList, AS_HASHABLE(DefaultFrameName), &hdata.vptr) == ASH_Success )
-				look->DefaultFrame = hdata.vptr ;
-            LOCAL_DEBUG_OUT( "DefaultFrameName is \"%s\": found frame %p with that name.", DefaultFrameName, look->DefaultFrame );
-        }
-    }else if( look->DefaultFrame != NULL )
-		destroy_myframe( &(look->DefaultFrame) );
-    if( look->DefaultFrame == NULL )
-        look->DefaultFrame = create_default_myframe(default_title_align|ALIGN_VCENTER);
+        LOCAL_DEBUG_OUT( "DefaultFrameName is \"%s\".", look->DefaultFrameName );
+    }
+
+	if( myframe_find( look->DefaultFrameName ) == NULL ) 
+	{	
+        MyFrame *dmf = create_default_myframe(default_title_align|ALIGN_VCENTER);
+		dmf->name = mystrdup(look->DefaultFrameName);
+		if( add_hash_item( look->FramesList, AS_HASHABLE(dmf->name), dmf ) != ASH_Success ) 
+			destroy_myframe( &dmf );
+	}
 
 #ifdef LOCAL_DEBUG
     LOCAL_DEBUG_OUT( "syncing %s","");
@@ -1089,21 +1172,18 @@ FixLook( MyLook *look )
     LOCAL_DEBUG_OUT( "syncing %s","");
     ASSync(False);
 #endif
-    /* updating balloons look */
-    if( BalloonConfig.style == NULL )
-        BalloonConfig.style = mystrdup( "TitleButtonBalloon" );
 
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
     Print_balloonConfig ( &BalloonConfig );
 #endif
-    balloon_config2look( look, &BalloonConfig );
+    balloon_config2look( look, &BalloonConfig, "TitleButtonBalloon" );
     set_balloon_look( look->balloon_look );
 
     /* checking sanity of the move-resize window geometry :*/
     if( (look->resize_move_geometry.flags&(HeightValue|WidthValue)) != (HeightValue|WidthValue))
 	{
-		int width = 0;
-		int height = 0 ;
+		unsigned int width = 0;
+		unsigned int height = 0 ;
 		get_text_size( " +88888 x +88888 ", look->MSWindow[BACK_FOCUSED]->font.as_font, look->MSWindow[BACK_FOCUSED]->text_style, &width, &height );
     	if( !get_flags(look->resize_move_geometry.flags, WidthValue ) )
 	        look->resize_move_geometry.width = width + SIZE_VINDENT * 2;
@@ -1119,6 +1199,7 @@ FixLook( MyLook *look )
         enable_hints_support( look->supported_hints, HINTS_ICCCM );
         enable_hints_support( look->supported_hints, HINTS_Motif );
         enable_hints_support( look->supported_hints, HINTS_Gnome );
+        enable_hints_support( look->supported_hints, HINTS_KDE );
         enable_hints_support( look->supported_hints, HINTS_ExtendedWM );
         enable_hints_support( look->supported_hints, HINTS_ASDatabase );
         enable_hints_support( look->supported_hints, HINTS_GroupLead );
@@ -1205,43 +1286,111 @@ redecorate_aswindow_iter_func(void *data, void *aux_data)
     ASWindow *asw = (ASWindow*)data;
     if(asw )
 	{
+		/* need to invalidate all MyStyles at this point ??? */
+		
+		invalidate_window_mystyles( asw );
+		
 		redecorate_window( asw, False );
         if( asw->internal && asw->internal->on_look_feel_changed )
             asw->internal->on_look_feel_changed( asw->internal, &Scr.Feel, &Scr.Look, ASFLAGS_EVERYTHING );
-        on_window_status_changed( asw, True, False );
+        on_window_status_changed( asw, True, True );
 		set_flags( asw->internal_flags, ASWF_PendingShapeRemoval );
     }
     return True;
 }
 
+void 
+advertise_tbar_props()
+{
+	ASTBarProps props ; 
+	MyFrame *frame = myframe_find(NULL);
+	MouseButton  *btn;
+	int i, k ; 
+
+#define TBAR_BUTTONS_OF_CONCERN 5
+	struct 
+	{
+		Atom *kind, *kind_down;
+	 	int func;
+		MyButton *btn;
+	}buttons[TBAR_BUTTONS_OF_CONCERN] = 
+	{	{ &_AS_BUTTON_CLOSE, 	&_AS_BUTTON_CLOSE_PRESSED, 		F_CLOSE, 	NULL },
+		{ &_AS_BUTTON_MAXIMIZE, &_AS_BUTTON_MAXIMIZE_PRESSED, 	F_MAXIMIZE, NULL },
+		{ &_AS_BUTTON_MINIMIZE, &_AS_BUTTON_MINIMIZE_PRESSED, 	F_ICONIFY, 	NULL },
+		{ &_AS_BUTTON_SHADE, 	&_AS_BUTTON_SHADE_PRESSED, 		F_SHADE, 	NULL },
+		{ &_AS_BUTTON_MENU, 	&_AS_BUTTON_MENU_PRESSED, 		F_POPUP, 	NULL }};
+
+	
+	props.align = frame->title_align ;
+	props.bevel = frame->title_fbevel|frame->title_ubevel ;
+	props.title_h_spacing = frame->title_h_spacing ;
+	props.title_v_spacing = frame->title_v_spacing ;
+	props.buttons_h_border = max(Scr.Look.TitleButtonXOffset[0],Scr.Look.TitleButtonXOffset[1]);
+	props.buttons_v_border = max(Scr.Look.TitleButtonYOffset[0],Scr.Look.TitleButtonYOffset[1]);
+	props.buttons_spacing = max(Scr.Look.TitleButtonSpacing[0],Scr.Look.TitleButtonSpacing[1]);
+		
+    for (btn = Scr.Feel.MouseButtonRoot; btn != NULL; btn = btn->NextButton)
+		if ( (btn->Context & C_TButtonAll) )
+		{
+			for( i = 0 ; i < TBAR_BUTTONS_OF_CONCERN ; ++i )
+			{
+				if( btn->fdata->func == buttons[i].func && buttons[i].btn == NULL )
+				{/* now lets find that button in look : */
+				 	for( k = 0  ; k < TITLE_BUTTONS ; ++k ) 
+						if ( Scr.Look.buttons[k].unpressed.image != NULL &&
+						     (btn->Context&Scr.Look.buttons[k].context) != 0 )
+						{
+							buttons[i].btn = &(Scr.Look.buttons[k]);
+							break;	
+						}
+					break;	
+				}	 
+			}
+		}
+	props.buttons_num = 0 ; 
+	for( i = 0 ; i < TBAR_BUTTONS_OF_CONCERN ; ++i )
+		if( buttons[i].btn != NULL ) 
+		{
+			++props.buttons_num	;
+			if( buttons[i].btn->pressed.image != NULL && 
+				buttons[i].btn->pressed.image != buttons[i].btn->unpressed.image )
+				++props.buttons_num	;
+		}	 
+	props.buttons = safemalloc( props.buttons_num*sizeof(struct ASButtonPropElem));
+	k = 0 ;
+	for( i = 0 ; i < TBAR_BUTTONS_OF_CONCERN ; ++i ) 
+		if( buttons[i].btn != NULL ) 
+		{
+			MyIcon *icon = &(buttons[i].btn->unpressed);
+			if( icon->pix == None ) 
+				make_icon_pixmaps( icon, False );
+	
+			props.buttons[k].kind = *(buttons[i].kind);
+			props.buttons[k].pmap = icon->pix;
+			props.buttons[k].mask = icon->mask;
+			props.buttons[k].alpha = icon->alpha;
+	   		++k ;
+			if( buttons[i].btn->pressed.image != NULL && 
+				buttons[i].btn->pressed.image != buttons[i].btn->unpressed.image )
+			{	  
+				icon = &(buttons[i].btn->pressed);
+				if( icon->pix == None ) 
+					make_icon_pixmaps( icon, False );
+				props.buttons[k].kind = *(buttons[i].kind_down);
+				props.buttons[k].pmap = icon->pix;
+				props.buttons[k].mask = icon->mask;
+				props.buttons[k].alpha = icon->alpha;
+				++k ;
+			}
+		}	 
+
+	set_astbar_props( Scr.wmprops, &props ); 
+}
+
 
 /*************************************************************************/
 /* reading confiug files now :                                           */
 /*************************************************************************/
-void
-ParseDatabase (const char *file)
-{
-    struct name_list *list = NULL ;
-
-	/* memory management for parsing buffer */
-	if (file == NULL)
-        return ;
-
-    list = ParseDatabaseOptions (file, "afterstep");
-    if( list )
-    {
-        Database = build_asdb( list );
-        if( is_output_level_under_threshold( OUTPUT_LEVEL_DATABASE ) )
-            print_asdb( NULL, NULL, Database );
-        while (list != NULL)
-            delete_name_list (&(list));
-    }else
-        show_progress( "no database records loaded." );
-    /* XResources : */
-    load_user_database();
-}
-
-
 int
 ParseConfigFile (const char *file, char **tline)
 {
@@ -1294,7 +1443,17 @@ LoadASConfig (int thisdesktop, ASFlagType what)
     char            *tline = NULL;
     ASImageManager  *old_image_manager = NULL ;
     ASFontManager   *old_font_manager  = NULL ;
-    
+	FunctionData gtkrc_signal_func ;
+	FunctionData kde_signal_func ;
+	ASEvent dummy_event = {0};
+	
+	init_func_data( &gtkrc_signal_func );
+    gtkrc_signal_func.func = F_SIGNAL_RELOAD_GTK_RCFILE ;
+	init_func_data( &kde_signal_func );
+    kde_signal_func.func = F_KIPC_SEND_MESSAGE_ALL ;
+	kde_signal_func.func_val[0] = KIPC_PaletteChanged ; 
+	kde_signal_func.func_val[1] = 0 ; 
+
     cover_desktop();
 
 #ifndef DIFFERENTLOOKNFEELFOREACHDESKTOP
@@ -1310,11 +1469,8 @@ LoadASConfig (int thisdesktop, ASFlagType what)
         const char *const_configfile;
         if (get_flags(what, PARSE_BASE_CONFIG))
 		{
-			if( ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, get_flags(what, PARSE_LOOK_CONFIG) ) )
+			if( ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, get_flags(what, PARSE_LOOK_CONFIG), True ) )
 			{
-				static char *PixmapPathEnvVar = NULL ; 
-				static char *FontPathEnvVar = NULL ; 
-				char *tmp ;
 				if( !get_flags(what, PARSE_LOOK_CONFIG) )
 				{
 					if( old_image_manager != NULL || old_font_manager != NULL )
@@ -1324,20 +1480,6 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 					}
 				}else
                 	clear_flags(what, PARSE_BASE_CONFIG);
-				if( Environment->pixmap_path != NULL ) 
-				{	
-					tmp = safemalloc(strlen( ASIMAGE_PATH_ENVVAR ) + 1 + strlen(Environment->pixmap_path)+1) ;
-					sprintf( tmp, "%s=%s", ASIMAGE_PATH_ENVVAR, Environment->pixmap_path );
-					putenv( tmp );
-					set_string_value( &PixmapPathEnvVar, tmp, NULL, 0 );
-				}
-				if( Environment->font_path ) 
-				{	
-					tmp = safemalloc(strlen( ASFONT_PATH_ENVVAR ) + 1 + strlen(Environment->font_path)+1) ;
-					sprintf( tmp, "%s=%s", ASFONT_PATH_ENVVAR, Environment->font_path );
-					putenv( tmp );
-					set_string_value( &FontPathEnvVar, tmp, NULL, 0 );
-				}
             }
         }else if(get_flags(what, PARSE_LOOK_CONFIG))
 		{  /* must reload Image manager so that changed images would get updated */
@@ -1354,8 +1496,11 @@ LoadASConfig (int thisdesktop, ASFlagType what)
             {
                 InitLook (&Scr.Look, True);
 				memset( &TmpLook, 0x00, sizeof(TmpLook));
+				TmpLook.magic = MAGIC_MYLOOK ;
 				InitLook (&TmpLook, False );
+				LOCAL_DEBUG_OUT( "desk_anime_tint = %lX", TmpLook.desktop_animation_tint );
                 ParseConfigFile (const_configfile, &tline);
+				LOCAL_DEBUG_OUT( "desk_anime_tint = %lX", TmpLook.desktop_animation_tint );
                 show_progress("LOOK configuration loaded from \"%s\" ...", const_configfile);
                 display_progress( True, "LOOK configuration loaded from \"%s\".", const_configfile);
 #ifdef ASETROOT_FILE
@@ -1380,7 +1525,11 @@ LoadASConfig (int thisdesktop, ASFlagType what)
                 display_progress( True, "LOOK configuration file cannot be found!");
                 clear_flags(what, PARSE_LOOK_CONFIG);
             }
-        }
+        	if( UpdateGtkRC(Environment) ) 
+				ExecuteFunction (&gtkrc_signal_func, &dummy_event, -1);
+			if( UpdateKCSRC() ) 
+				ExecuteFunction (&kde_signal_func, &dummy_event, -1);
+		}
         if (get_flags(what, PARSE_FEEL_CONFIG))
 		{
             if( (const_configfile = get_session_file (Session, thisdesktop, F_CHANGE_FEEL, False) ) != NULL )
@@ -1391,7 +1540,12 @@ LoadASConfig (int thisdesktop, ASFlagType what)
                 InitFeel (&Scr.Feel, True);
                 if (tline == NULL)
                     tline = safemalloc (MAXLINELENGTH + 1);
-                display_progress( True, "Parsing menu entries and checking availability ...");
+				
+				display_progress( True, "Reloading and merging desktop categories ...");
+				ReloadCategories(False);
+				UpdateCategoriesCache();
+                
+				display_progress( True, "Parsing menu entries and checking availability ...");
                 MeltStartMenu (tline);
                 display_progress( False, "Done..");
                 ParseConfigFile (const_configfile, &tline);
@@ -1444,24 +1598,21 @@ LoadASConfig (int thisdesktop, ASFlagType what)
         }
 
         if (get_flags(what, PARSE_DATABASE_CONFIG))
-        {
-            if( (configfile = make_session_file(Session, DATABASE_FILE, False )) != NULL )
-            {
-                InitDatabase (True);
-                ParseDatabase (configfile);
-                show_progress("DATABASE configuration loaded from \"%s\" ...", configfile);
-                display_progress( True, "DATABASE configuration loaded from \"%s\" .", configfile);
-                free( configfile );
-            }else
-            {
-                show_warning("DATABASE configuration file cannot be found!");
-                display_progress( True, "DATABASE configuration file cannot be found!");
+		{
+			if( !ReloadASDatabase() )
+			{
+		        display_progress( True, "DATABASE configuration file cannot be found!");
                 clear_flags(what, PARSE_DATABASE_CONFIG);
-            }
-        }
+			}else
+			{
+				configfile  = make_session_file(Session, DATABASE_FILE, False );
+	    	    display_progress( True, "DATABASE configuration loaded from \"%s\" .", configfile);
+				free( configfile );
+			}
+		}
 	} else
 	{
-		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True );
+		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True, True );
 		LoadColorScheme();
 		memset( &TmpLook, 0x00, sizeof(TmpLook));
 		InitLook (&TmpLook, False );
@@ -1469,13 +1620,16 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 		memset( &TmpFeel, 0x00, sizeof(TmpFeel));
 		InitFeel (&TmpFeel, False );
         InitFeel (&Scr.Feel, True);
-        InitDatabase (True);
         ParseConfigFile (Session->overriding_file, &tline);
+        ReloadASDatabase();
 		merge_look( &Scr.Look, &TmpLook );
 		merge_feel( &Scr.Feel, &TmpFeel );
         show_progress("AfterStep configuration loaded from \"%s\" ...", Session->overriding_file);
         display_progress( True, "AfterStep configuration loaded from \"%s\".", Session->overriding_file);
         what = PARSE_EVERYTHING ;
+		if( UpdateGtkRC(Environment) ) 
+			ExecuteFunction (&gtkrc_signal_func, &dummy_event, -1);
+		UpdateKCSRC();
     }
 
 	/* let's free the memory used for parsing */
@@ -1509,12 +1663,14 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 		}
     }
 
-    if( get_flags(what, PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
+    if( get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
 	{
+        ASHashIterator  i;
 		ARGB32 cursor_fore = ARGB32_White ;
 		ARGB32 cursor_back = ARGB32_Black ;
 
 		fix_menu_pin_on( &Scr.Look );
+		
 		/* also need to recolor cursors ! */
 		if( Scr.Look.CursorFore )
 			parse_argb_color( Scr.Look.CursorFore, &cursor_fore );
@@ -1522,23 +1678,16 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 	   		parse_argb_color( Scr.Look.CursorBack, &cursor_back );
 		recolor_feel_cursors( &Scr.Feel, cursor_fore, cursor_back );
 		XDefineCursor (dpy, Scr.Root, Scr.Feel.cursors[ASCUR_Default]);
-	}
-	if( get_flags(what, PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
-	{
-        ASHashIterator  i;
-        if( start_hash_iteration (Scr.Feel.Popups, &i) )
+        
+		if( start_hash_iteration (Scr.Feel.Popups, &i) )
             do
             {
 				reload_menu_pmaps( (MenuData *)curr_hash_data(&i) );
             }while( next_hash_item( &i ));
+		
+		advertise_tbar_props();
 	}
 	   
-    /* TODO: update the menus */
-    if (get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
-	{
-		/* TODO: update mouse cursor ! */
-    }
-
     /* force update of window frames */
     if (get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG|PARSE_DATABASE_CONFIG))
     {
@@ -1566,8 +1715,10 @@ LoadASConfig (int thisdesktop, ASFlagType what)
     }
 
     ConfigureNotifyLoop();
+
     remove_desktop_cover();
 
+	validate_rootpmap_props(Scr.wmprops);
 }
 
 /*****************************************************************************
@@ -1739,7 +1890,8 @@ SetCustomCursor (char *text, FILE * fd, char **arg, int *junk)
 	int           num, cursor_num;
 	char          f_cursor[1024], f_mask[1024];
 	Pixmap        cursor = None, mask = None;
-	int           width, height, x, y;
+	unsigned int  width, height;
+	int x, y;
 	XColor        fore, back;
 	char         *path;
     Cursor new_c ;
