@@ -362,8 +362,15 @@ DigestEvent( ASEvent *event )
 			event->client = NULL ;
 		}else
 		{
-			event->widget = NULL ;	
-			event->client = window2ASWindow( event->w );
+			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+			{	
+				event->client = NULL ;
+				event->widget = NULL ;	 
+			}else
+			{	
+				event->widget = NULL ;	
+				event->client = window2ASWindow( event->w );
+			}
 		}	 
 	}
 
@@ -456,6 +463,11 @@ DigestEvent( ASEvent *event )
                         }
                 }
             }
+			if( event->context == C_NO_CONTEXT && get_flags(Scr.Feel.flags, ClickToFocus) )
+			{
+				w = asw->frame ; 
+				event->context = C_FRAME ;
+			}	 
             event->w = w ;
         }else
         {
@@ -749,7 +761,7 @@ HandleKeyPress ( ASEvent *event )
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         EndWarping();
 
-	LOCAL_DEBUG_OUT( "client = %p", event->client );
+	LOCAL_DEBUG_OUT( "client = %p, context = %s", event->client, context2text((event)->context));
 	/* if we get here, no function key was bound to the key.  Send it
      * to the client if it was in a window we know about: */
     if (event->client)
@@ -808,7 +820,7 @@ HandlePropertyNotify (ASEvent *event)
         }
         if(Scr.RootBackground && Scr.RootBackground->im != NULL )
         {
-            if( Scr.RootBackground->pmap && Scr.wmprops->root_pixmap == Scr.RootBackground->pmap );
+            if( Scr.RootBackground->pmap && Scr.wmprops->root_pixmap == Scr.RootBackground->pmap )
                 Scr.RootImage = dup_asimage(Scr.RootBackground->im) ;
         }
 		if( Scr.wmprops->as_root_pixmap != Scr.wmprops->root_pixmap ) 
@@ -899,7 +911,7 @@ HandleClientMessage (ASEvent *event)
         (event->x.xclient.data.l[0] == IconicState) &&
         !ASWIN_GET_FLAGS(event->client, AS_Iconic))
 	{
-        set_window_wm_state( event->client, True );
+        set_window_wm_state( event->client, True, False );
 #ifdef ENABLE_DND
 		/* Pass the event to the client window */
         if (event->x.xclient.window != event->client->w)
@@ -970,10 +982,10 @@ HandleMapRequest (ASEvent *event )
     /* If the window has never been mapped before ... */
     if (event->client == NULL)
     {
-        if( (event->client = AddWindow (event->w)) == NULL )
+        if( (event->client = AddWindow (event->w, True)) == NULL )
             return;
     }else /* If no hints, or currently an icon, just "deiconify" */
-        set_window_wm_state( event->client, False );
+        set_window_wm_state( event->client, False, True );
 }
 
 /***********************************************************************
@@ -992,6 +1004,7 @@ HandleMapNotify ( ASEvent *event )
     if ( asw == NULL || event->w == Scr.Root )
         return;
 
+    LOCAL_DEBUG_OUT( "asw->w = %lX, event->w = %lX", asw->w, event->w );
     if( event->w != asw->w )
     {
         if( asw->wm_state_transition == ASWT_Withdrawn2Iconic && event->w == asw->status->icon_window )
@@ -1002,11 +1015,18 @@ HandleMapNotify ( ASEvent *event )
     }
 
 	if( asw->wm_state_transition == ASWT_Withdrawn2Normal )
-		no_focus = (ASWIN_HFLAGS(asw, AS_FocusOnMap) == 0) ;
-    if( asw->wm_state_transition == ASWT_StableState )
+	{
+		if( ASWIN_HFLAGS(asw, AS_FocusOnMap) )
+			force_activation = True;
+		else
+			no_focus = True ;
+	}
+    LOCAL_DEBUG_OUT( "asw->wm_state_transition = %d", asw->wm_state_transition );
+
+	if( asw->wm_state_transition == ASWT_StableState )
     {
         if( ASWIN_GET_FLAGS( asw, AS_Iconic ) )
-            set_window_wm_state( asw, False );  /* client has requested deiconification */
+            set_window_wm_state( asw, False, False );  /* client has requested deiconification */
         return ;                                /* otherwise it is redundand event */
     }
     if( get_flags(asw->wm_state_transition, ASWT_FROM_ICONIC ) )
@@ -1017,7 +1037,8 @@ HandleMapNotify ( ASEvent *event )
     ASWIN_CLEAR_FLAGS(asw, AS_IconMapped);
     ASWIN_CLEAR_FLAGS(asw, AS_Iconic);
     complete_wm_state_transition( asw, NormalState );
-	if( !no_focus )
+	LOCAL_DEBUG_OUT( "no_focus = %d, force_activation = %d, AcceptsFocus = %ld", no_focus, force_activation, ASWIN_HFLAGS(asw,AS_AcceptsFocus) );
+	if( !no_focus && ASWIN_HFLAGS(asw,AS_AcceptsFocus) )
     	activate_aswindow (asw, force_activation, False);
     broadcast_config( M_MAP, asw );
     /* finally reaches Normal state */
@@ -1071,12 +1092,12 @@ HandleUnmapNotify (ASEvent *event )
 
     grab_server();
     destroyed = ASCheckTypedWindowEvent ( event->w, DestroyNotify, &dummy) ;
-	LOCAL_DEBUG_OUT("wm_state_transition = 0x%X", event->client->wm_state_transition );
-	if( !get_flags( event->client->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
-    	event->client->wm_state_transition = ASWIN_GET_FLAGS(event->client, AS_Iconic)?ASWT_Iconic2Withdrawn:ASWT_Normal2Withdrawn ;
+	LOCAL_DEBUG_OUT("wm_state_transition = 0x%X", asw->wm_state_transition );
+	if( !get_flags( asw->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
+    	asw->wm_state_transition = ASWIN_GET_FLAGS(asw, AS_Iconic)?ASWT_Iconic2Withdrawn:ASWT_Normal2Withdrawn ;
 	else
-		event->client->wm_state_transition = ASWT_Withdrawn2Withdrawn ;
-    Destroy (event->client, destroyed);               /* do not need to mash event before */
+		asw->wm_state_transition = ASWT_Withdrawn2Withdrawn ;
+    Destroy (asw, destroyed);               /* do not need to mash event before */
     ungrab_server();
     ASFlush ();
 }
@@ -1325,7 +1346,8 @@ HandleConfigureRequest ( ASEvent *event )
 		return;
 	}
 
-	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) ||
+		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
 	{	
 		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
 		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas);
