@@ -886,7 +886,7 @@ on_window_hints_changed( ASWindow *asw )
         return ;
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_hints( NULL, NULL, &raw_hints );
-    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL );
+    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
     destroy_raw_hints( &raw_hints, True );
     if( hints )
     {
@@ -920,10 +920,11 @@ on_window_hints_changed( ASWindow *asw )
 	if( hints2decorations( asw, old_hints ) )
 	    on_window_status_changed( asw, False, True );
 
+	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 || 
+		mystrcmp( old_hints->res_class, hints->res_class ) != 0)
+  		broadcast_res_names( asw );
 	if( mystrcmp( old_hints->names[0], hints->names[0] ) != 0 )
 	    broadcast_window_name( asw );
-	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 )
-  		broadcast_res_names( asw );
 	if( mystrcmp( old_hints->icon_name, hints->icon_name ) != 0 )
 	    broadcast_icon_name( asw );
 
@@ -1258,12 +1259,12 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s)", asw, context2text(pressed_context));
 void
 save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 {
-    if( hor && asw->saved_anchor.width == 0 )
+    if( hor && asw->saved_anchor.width == 0 && asw->status->width < Scr.MyDisplayWidth )
     {
         asw->saved_anchor.x = asw->anchor.x ;
         asw->saved_anchor.width = asw->anchor.width ;
     }
-    if( vert && asw->saved_anchor.height == 0 )
+    if( vert && asw->saved_anchor.height == 0 && asw->status->height < Scr.MyDisplayHeight  )
     {
         asw->saved_anchor.y = asw->anchor.y ;
         asw->saved_anchor.height = asw->anchor.height ;
@@ -1359,52 +1360,14 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 		t->status->viewport_x = Scr.Vx ;
 		t->status->viewport_y = Scr.Vy ;
 	}
-	if (!get_flags (t->status->flags, AS_Sticky))
+	if (!get_flags (t->status->flags, AS_Sticky) && 
+		!ASWIN_HFLAGS(t, AS_UseCurrentViewport))
 	{
 		t->status->x -= t->status->viewport_x;
 		t->status->y -= t->status->viewport_y;
 	}
 	LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-
-    if( !get_flags(AfterStepState, ASS_NormalOperation) )
-    {
-        int min_x, min_y, max_x, max_y ;
-        int margin = Scr.MyDisplayWidth>>5 ;
-        if (!ASWIN_GET_FLAGS( t, AS_Sticky ))
-        {
-            min_x = -Scr.Vx ;
-            max_x = Scr.VxMax+Scr.MyDisplayWidth ;
-            min_y = -Scr.Vy ;
-            max_y = Scr.VyMax+Scr.MyDisplayHeight ;
-        }else
-        {
-            min_x = 0 ;
-            max_x = Scr.MyDisplayWidth ;
-            min_y = 0 ;
-            max_y = Scr.MyDisplayHeight ;
-        }
-        /* we have to make sure that window is visible !!!! */
-		LOCAL_DEBUG_OUT( "x_range(%d,%d), y_range(%d,%d), margin = %d", min_x, max_x, min_y, max_y, margin );
-        if( (int)t->status->x + (int)t->status->width < min_x + margin )
-            t->status->x = min_x + margin - (int)t->status->width ;
-        else if( (int)t->status->x > max_x-margin )
-            t->status->x = max_x - margin ;
-        if( (int)t->status->y + (int)t->status->height < min_y + margin )
-            t->status->y = min_y + margin - (int)t->status->height ;
-        else if( (int)t->status->y > max_y-margin )
-            t->status->y = max_y - margin ;
-
-		LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-
-        set_flags( t->status->flags, AS_Position );
-
-    }else if( get_flags(Scr.Feel.flags, NoPPosition))
-	{ 
-      	if( !get_flags( t->hints->flags, AS_Transient ) && 
-		    !get_flags( status->flags, AS_StartPositionUser ) )
-	        clear_flags( t->status->flags, AS_Position );
-	}
-    
+	
 	/* TODO: AS_Iconic */
 	if( !ASWIN_GET_FLAGS(t, AS_StartLayer ) )
         ASWIN_LAYER(t) = AS_LayerNormal ;
@@ -1413,39 +1376,118 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
     else if( ASWIN_LAYER(t) > AS_LayerHighest )
         ASWIN_LAYER(t) = AS_LayerHighest ;
 
-    if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
-        pending_placement = True ;
-    else if( !get_flags( t->status->flags, AS_Position ))
-    {
-        if( ! get_flags( t->status->flags, AS_StartsIconic ) )
-        {
-            int x = -1, y = -1;
-			pending_placement = True ;
-            ASQueryPointerRootXY( &x, &y );
-	        if( get_flags( t->hints->flags, AS_Transient|AS_ShortLived ) )
+	if( get_flags( t->status->flags, AS_Fullscreen ))
+	{
+        t->status->width = Scr.MyDisplayWidth ;
+        t->status->height = Scr.MyDisplayHeight ;
+        t->status->x = 0 ;
+        t->status->y = 0 ;
+	}else
+	{	
+		if( get_flags (t->hints->flags, AS_MinSize) ) 
+		{	
+			int width = t->status->width ;
+			int height = t->status->height ;
+			if( (!get_flags (t->status->flags, AS_StartSizeUser) && width < t->hints->min_width) || width == 1 )
+				width = min( t->hints->min_width, Scr.MyDisplayWidth );
+			if( (!get_flags (t->status->flags, AS_StartSizeUser) && height < t->hints->min_height) || height == 1 )
+				width = min( t->hints->min_height, Scr.MyDisplayHeight );
+			if( width != t->status->width || height != t->status->height ) 
 			{
-				x -= (int)t->status->width / 2 ;
-				y -= (int)t->status->height / 2 ;
-				set_flags( t->status->flags, AS_Position );
-				pending_placement = False ;
-			}
-
-			if( x + (int)t->status->width > (int)Scr.MyDisplayWidth )
-				x = (int)Scr.MyDisplayWidth - (int)t->status->width ;
-			if( x < 0 )
-				x = 0 ;
-
-			if( y + (int)t->status->height > (int) Scr.MyDisplayHeight )
-				y = Scr.MyDisplayHeight - (int)t->status->height ;
-			if( y < 0 ) 
-				y = 0 ;
-
-            t->status->x = x ;
-            t->status->y = y ;
+				int dx = 0, dy = 0 ;
+				if( t->hints->gravity == EastGravity || 
+					t->hints->gravity == SouthEastGravity || 
+					t->hints->gravity == NorthEastGravity ||
+					t->hints->gravity == CenterGravity )
+					dx = (int)(t->status->width) - width ; 
+				if( t->hints->gravity == SouthGravity || 
+					t->hints->gravity == SouthEastGravity || 
+					t->hints->gravity == SouthWestGravity ||
+					t->hints->gravity == CenterGravity)
+					dy = (int)(t->status->height) - height ; 
+				if( t->hints->gravity == EastGravity || t->hints->gravity == CenterGravity ) 
+					dx = dx/2 ;
+				if( t->hints->gravity == SouthGravity || t->hints->gravity == CenterGravity ) 
+					dy = dy/2 ;
+				t->status->x += dx ;			
+				t->status->y += dy ;			   
+				t->status->width = width ;			   
+				t->status->height = height ;			   
+				XResizeWindow( dpy, t->w, width, height );
+			}	 
+		}
+    	if( !get_flags(AfterStepState, ASS_NormalOperation) )
+    	{
+        	int min_x, min_y, max_x, max_y ;
+        	int margin = Scr.MyDisplayWidth>>5 ;
+        	if (!ASWIN_GET_FLAGS( t, AS_Sticky ))
+        	{
+            	min_x = -Scr.Vx ;
+            	max_x = Scr.VxMax+Scr.MyDisplayWidth ;
+            	min_y = -Scr.Vy ;
+            	max_y = Scr.VyMax+Scr.MyDisplayHeight ;
+        	}else
+        	{
+            	min_x = 0 ;
+            	max_x = Scr.MyDisplayWidth ;
+            	min_y = 0 ;
+            	max_y = Scr.MyDisplayHeight ;
+        	}
+        	/* we have to make sure that window is visible !!!! */
+			LOCAL_DEBUG_OUT( "x_range(%d,%d), y_range(%d,%d), margin = %d", min_x, max_x, min_y, max_y, margin );
+        	if( (int)t->status->x + (int)t->status->width < min_x + margin )
+            	t->status->x = min_x + margin - (int)t->status->width ;
+        	else if( (int)t->status->x > max_x-margin )
+            	t->status->x = max_x - margin ;
+        	if( (int)t->status->y + (int)t->status->height < min_y + margin )
+            	t->status->y = min_y + margin - (int)t->status->height ;
+        	else if( (int)t->status->y > max_y-margin )
+            	t->status->y = max_y - margin ;
 
 			LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-        }
-    }
+
+        	set_flags( t->status->flags, AS_Position );
+
+    	}else if( get_flags(Scr.Feel.flags, NoPPosition))
+		{ 
+      		if( !get_flags( t->hints->flags, AS_Transient ) && 
+		    	!get_flags( status->flags, AS_StartPositionUser ) )
+	        	clear_flags( t->status->flags, AS_Position );
+		}
+		 if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
+        	pending_placement = True ;
+		else  if( !get_flags( t->status->flags, AS_Position ))
+    	{
+        	if( ! get_flags( t->status->flags, AS_StartsIconic ) )
+        	{
+            	int x = -1, y = -1;
+				pending_placement = True ;
+            	ASQueryPointerRootXY( &x, &y );
+	        	if( get_flags( t->hints->flags, AS_Transient|AS_ShortLived ) )
+				{
+					x -= (int)t->status->width / 2 ;
+					y -= (int)t->status->height / 2 ;
+					set_flags( t->status->flags, AS_Position );
+					pending_placement = False ;
+				}
+
+				if( x + (int)t->status->width > (int)Scr.MyDisplayWidth )
+					x = (int)Scr.MyDisplayWidth - (int)t->status->width ;
+				if( x < 0 )
+					x = 0 ;
+
+				if( y + (int)t->status->height > (int) Scr.MyDisplayHeight )
+					y = Scr.MyDisplayHeight - (int)t->status->height ;
+				if( y < 0 ) 
+					y = 0 ;
+
+            	t->status->x = x ;
+            	t->status->y = y ;
+
+				LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
+        	}
+    	}
+	}		
 
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_status_hints( NULL, NULL, t->status );
@@ -1490,7 +1532,7 @@ complete_wm_state_transition( ASWindow *asw, int state )
 }
 
 Bool
-set_window_wm_state( ASWindow *asw, Bool iconify )
+set_window_wm_state( ASWindow *asw, Bool iconify, Bool force_unmapped )
 {
     XWindowAttributes attr;
 
@@ -1571,20 +1613,21 @@ LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_N
             if( asw->icon_canvas != asw->icon_title_canvas )
                 unmap_canvas_window(asw->icon_title_canvas );
 
-            ASWIN_DESK(asw) = get_flags(Scr.Feel.flags, StubbornIcons)?asw->DeIconifyDesk:Scr.CurrentDesk;
-            quietly_reparent_aswindow( asw, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, True );
+			change_aswindow_desktop( asw, get_flags(Scr.Feel.flags, StubbornIcons)?asw->DeIconifyDesk:Scr.CurrentDesk, True );
         }
 
         asw->status->icon_window = None ;
 
-        if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+        if( !ASWIN_GET_FLAGS(asw, AS_Dead) && !force_unmapped )
         {
             if ( !XGetWindowAttributes (dpy, asw->w, &attr) )
                 ASWIN_SET_FLAGS(asw, AS_Dead);
-            else
+		}
+        if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+        {
             {
                 /* TODO: make sure that the window is on this screen */
-                if( attr.map_state == IsUnmapped )
+                if( attr.map_state == IsUnmapped || force_unmapped)
                     XMapRaised (dpy, asw->w);
                 else
                 {
@@ -1673,7 +1716,7 @@ make_aswindow_visible( ASWindow *asw, Bool deiconify )
     {
         if( deiconify )
         {
-			set_window_wm_state( asw, False );
+			set_window_wm_state( asw, False, False );
 		}
     }
 
@@ -1739,8 +1782,8 @@ make_aswindow_visible( ASWindow *asw, Bool deiconify )
         }else
         {
             on_window_moveresize( asw, asw->frame );
-            x = asw->frame_canvas->root_x;
-            y = asw->frame_canvas->root_y;
+            x = asw->client_canvas->root_x;
+            y = asw->client_canvas->root_y;
         }
 		LOCAL_DEBUG_OUT( "Warping pointer to : %+d%+d", x + Scr.Feel.Xzap, y + Scr.Feel.Yzap);
         XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, x + Scr.Feel.Xzap, y + Scr.Feel.Yzap);
@@ -1765,12 +1808,15 @@ LOCAL_DEBUG_OUT( "changing window's layer to %d", layer );
     }
 }
 
-void change_aswindow_desktop( ASWindow *asw, int new_desk )
+void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
 {
     int old_desk ;
     if( AS_ASSERT(asw) )
         return ;
-    if( ASWIN_DESK(asw) == new_desk || ASWIN_GET_FLAGS(asw, AS_Sticky))
+	if( ASWIN_GET_FLAGS(asw, AS_Sticky) ) 
+		new_desk = Scr.CurrentDesk ;
+    
+	if( !force && ASWIN_DESK(asw) == new_desk )
         return ;
 
     old_desk = ASWIN_DESK(asw) ;
@@ -1789,6 +1835,31 @@ void change_aswindow_desktop( ASWindow *asw, int new_desk )
     }
 }
 
+static Bool
+restore_anchor_x( ASWindow *asw )
+{
+	if( asw->saved_anchor.width > 0)
+    {
+    	asw->anchor.x = asw->saved_anchor.x ;
+        asw->anchor.width = asw->saved_anchor.width ;
+        asw->saved_anchor.width = 0 ;  /* invalidating saved anchor */
+		return True;
+	}
+	return False;
+}
+static Bool
+restore_anchor_y( ASWindow *asw )
+{
+    if( asw->saved_anchor.height > 0)
+    {
+        asw->anchor.y = asw->saved_anchor.y ;
+        asw->anchor.height = asw->saved_anchor.height ;
+        asw->saved_anchor.height = 0 ; /* invalidating saved anchor */
+        return True ;
+	}
+	return False;	  
+}	  
+
 void toggle_aswindow_status( ASWindow *asw, ASFlagType flags )
 {
     ASFlagType on_flags, off_flags ;
@@ -1797,9 +1868,13 @@ void toggle_aswindow_status( ASWindow *asw, ASFlagType flags )
 
     if( AS_ASSERT(asw) )
         return ;
+
     if( flags == 0 )
         return ;
 
+	if( get_flags( flags, AS_Fullscreen ) )
+		clear_flags( flags, AS_MaximizedX|AS_MaximizedY);
+
     on_flags = (~(asw->status->flags))&flags ;
     off_flags = (asw->status->flags)&(~flags) ;
     asw->status->flags = on_flags|off_flags ;
@@ -1834,32 +1909,56 @@ LOCAL_DEBUG_OUT( "flags = %lx, on_flags = %lx, off_flags = %lx", flags, on_flags
 		}
 	}
 
+	if( get_flags( flags, AS_Fullscreen) )
+	{
+		if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )			   
+		{	                   /* fullscreen->normal */
+			if( !ASWIN_GET_FLAGS( asw, AS_MaximizedX ) )			
+	            if( !restore_anchor_x( asw ) )
+				{
+					asw->anchor.width = Scr.MyDisplayWidth/3;
+					asw->anchor.x = (Scr.MyDisplayWidth - asw->anchor.width) / 2;
+				}
+
+	        if( !ASWIN_GET_FLAGS( asw, AS_MaximizedY ) )
+	            if(!restore_anchor_y( asw ))
+				{
+					asw->anchor.height = Scr.MyDisplayHeight/3;
+					asw->anchor.y = (Scr.MyDisplayHeight - asw->anchor.height) / 2;
+				}
+
+			reconfigured = True;
+			if( ASWIN_GET_FLAGS( asw, AS_MaximizedY|AS_MaximizedX ) )
+				need_placement = True ;	
+		}else
+		{ 	                   /* normal->fullscreen */
+			ASStatusHints scratch_status = *(asw->status) ;
+			scratch_status.x = 0 ;
+			scratch_status.y = 0 ;
+			scratch_status.width = Scr.MyDisplayWidth ;
+			scratch_status.height = Scr.MyDisplayHeight ;
+	 		
+			save_aswindow_anchor( asw, True, True );		   
+			status2anchor(&(asw->anchor), asw->hints, &scratch_status, Scr.VxMax, Scr.VyMax);
+			reconfigured = True ;
+		}
+	}
     if( get_flags( flags, AS_MaximizedX) )
     {
         if( !ASWIN_GET_FLAGS( asw, AS_MaximizedX ) )
-        {
-            if( asw->saved_anchor.width > 0)
-            {
-                asw->anchor.x = asw->saved_anchor.x ;
-                asw->anchor.width = asw->saved_anchor.width ;
-                asw->saved_anchor.width = 0 ;  /* invalidating saved anchor */
-                reconfigured = True ;
-            }
-        }else
+		{	
+            if( restore_anchor_x( asw ) ) 
+				reconfigured = True ;
+        }else if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
             need_placement = True ;
     }
     if( get_flags( flags, AS_MaximizedY) )
     {
         if( !ASWIN_GET_FLAGS( asw, AS_MaximizedY ) )
         {
-            if( asw->saved_anchor.height > 0)
-            {
-                asw->anchor.y = asw->saved_anchor.y ;
-                asw->anchor.height = asw->saved_anchor.height ;
-                asw->saved_anchor.height = 0 ; /* invalidating saved anchor */
-                reconfigured = True ;
-            }
-        }else
+            if(restore_anchor_y( asw ))
+				reconfigured = True;
+        }else if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
             need_placement = True ;
     }
 
@@ -1894,7 +1993,7 @@ LOCAL_DEBUG_OUT("current focused is %p, active is %p", Scr.Windows->focused, Scr
         {
 LOCAL_DEBUG_OUT( "Window is iconic - pending implementation%s","");
             if( deiconify )
-				set_window_wm_state( asw, False );
+				set_window_wm_state( asw, False, False );
             else
                 return False;
         }
